"""
Settings resource for organization and unsubscribe settings.
"""

from typing import List, Optional

from emailr.http import AsyncHttpClient, HttpClient
from emailr.types import (
    Organization,
    UpdateOrganizationRequest,
    TeamMember,
    UnsubscribeSettings,
    UpdateUnsubscribeSettingsRequest,
)


class SettingsResource:
    """Synchronous settings resource."""

    def __init__(self, http: HttpClient) -> None:
        self._http = http

    def get_organization(self) -> Organization:
        """
        Get organization settings.

        Returns:
            Organization settings
        """
        data = self._http.get("/v1/settings/organization")
        return Organization.from_dict(data)

    def update_organization(
        self,
        *,
        name: Optional[str] = None,
        billing_email: Optional[str] = None,
    ) -> Organization:
        """
        Update organization settings.

        Args:
            name: Organization name
            billing_email: Billing email address

        Returns:
            Updated organization
        """
        request = UpdateOrganizationRequest(name=name, billing_email=billing_email)
        data = self._http.put("/v1/settings/organization", request.to_dict())
        return Organization.from_dict(data)

    def get_team(self) -> List[TeamMember]:
        """
        Get team members.

        Returns:
            List of team members
        """
        data = self._http.get("/v1/settings/team")
        if isinstance(data, list):
            return [TeamMember.from_dict(m) for m in data]
        return [TeamMember.from_dict(m) for m in data.get("data", data)]

    def get_unsubscribe_settings(self) -> UnsubscribeSettings:
        """
        Get unsubscribe settings.

        Returns:
            Unsubscribe settings
        """
        data = self._http.get("/v1/settings/unsubscribe")
        return UnsubscribeSettings.from_dict(data)

    def update_unsubscribe_settings(
        self,
        *,
        custom_message: Optional[str] = None,
        redirect_url: Optional[str] = None,
        one_click_enabled: Optional[bool] = None,
        logo_url: Optional[str] = None,
        primary_color: Optional[str] = None,
        company_name: Optional[str] = None,
    ) -> UnsubscribeSettings:
        """
        Update unsubscribe settings.

        Args:
            custom_message: Custom unsubscribe message
            redirect_url: Redirect URL after unsubscribe
            one_click_enabled: Enable one-click unsubscribe
            logo_url: Logo URL for branding
            primary_color: Primary color (hex format)
            company_name: Company name for branding

        Returns:
            Updated unsubscribe settings
        """
        request = UpdateUnsubscribeSettingsRequest(
            custom_message=custom_message,
            redirect_url=redirect_url,
            one_click_enabled=one_click_enabled,
            logo_url=logo_url,
            primary_color=primary_color,
            company_name=company_name,
        )
        data = self._http.put("/v1/settings/unsubscribe", request.to_dict())
        return UnsubscribeSettings.from_dict(data)


class AsyncSettingsResource:
    """Asynchronous settings resource."""

    def __init__(self, http: AsyncHttpClient) -> None:
        self._http = http

    async def get_organization(self) -> Organization:
        """Get organization settings."""
        data = await self._http.get("/v1/settings/organization")
        return Organization.from_dict(data)

    async def update_organization(
        self,
        *,
        name: Optional[str] = None,
        billing_email: Optional[str] = None,
    ) -> Organization:
        """Update organization settings."""
        request = UpdateOrganizationRequest(name=name, billing_email=billing_email)
        data = await self._http.put("/v1/settings/organization", request.to_dict())
        return Organization.from_dict(data)

    async def get_team(self) -> List[TeamMember]:
        """Get team members."""
        data = await self._http.get("/v1/settings/team")
        if isinstance(data, list):
            return [TeamMember.from_dict(m) for m in data]
        return [TeamMember.from_dict(m) for m in data.get("data", data)]

    async def get_unsubscribe_settings(self) -> UnsubscribeSettings:
        """Get unsubscribe settings."""
        data = await self._http.get("/v1/settings/unsubscribe")
        return UnsubscribeSettings.from_dict(data)

    async def update_unsubscribe_settings(
        self,
        *,
        custom_message: Optional[str] = None,
        redirect_url: Optional[str] = None,
        one_click_enabled: Optional[bool] = None,
        logo_url: Optional[str] = None,
        primary_color: Optional[str] = None,
        company_name: Optional[str] = None,
    ) -> UnsubscribeSettings:
        """Update unsubscribe settings."""
        request = UpdateUnsubscribeSettingsRequest(
            custom_message=custom_message,
            redirect_url=redirect_url,
            one_click_enabled=one_click_enabled,
            logo_url=logo_url,
            primary_color=primary_color,
            company_name=company_name,
        )
        data = await self._http.put("/v1/settings/unsubscribe", request.to_dict())
        return UnsubscribeSettings.from_dict(data)



"""
SMTP resource for getting SMTP credentials.
"""

from emailr.http import AsyncHttpClient, HttpClient
from emailr.types import SmtpCredentials


class SmtpResource:
    """Synchronous SMTP resource."""

    def __init__(self, http: HttpClient) -> None:
        self._http = http

    def get_credentials(self) -> SmtpCredentials:
        """
        Get SMTP credentials for sending emails.
        The password is your API key.

        Returns:
            SMTP credentials
        """
        data = self._http.get("/v1/smtp/credentials")
        return SmtpCredentials.from_dict(data)


class AsyncSmtpResource:
    """Asynchronous SMTP resource."""

    def __init__(self, http: AsyncHttpClient) -> None:
        self._http = http

    async def get_credentials(self) -> SmtpCredentials:
        """Get SMTP credentials for sending emails."""
        data = await self._http.get("/v1/smtp/credentials")
        return SmtpCredentials.from_dict(data)



"""
Domain resource for managing email domains.
"""

from typing import Any, Dict, List, Optional

from emailr.http import AsyncHttpClient, HttpClient
from emailr.types import (
    AddDomainRequest,
    DnsVerificationStatus,
    Domain,
    DomainVerificationStatus,
    PaginatedResponse,
    Pagination,
    SuccessResponse,
    UpdateDomainRequest,
)


class DomainsResource:
    """Synchronous domain resource for managing email domains."""

    def __init__(self, http: HttpClient) -> None:
        self._http = http

    def add(
        self,
        domain: str,
        *,
        receiving_subdomain: Optional[str] = None,
    ) -> Domain:
        """
        Add a new domain.

        Args:
            domain: Domain name to add
            receiving_subdomain: Subdomain for receiving emails

        Returns:
            Created domain
        """
        request = AddDomainRequest(domain=domain, receiving_subdomain=receiving_subdomain)
        data = self._http.post("/v1/domains", request.to_dict())
        return Domain.from_dict(data)

    def get(self, id: str) -> Domain:
        """
        Get domain by ID.

        Args:
            id: Domain ID

        Returns:
            Domain record
        """
        data = self._http.get(f"/v1/domains/{id}")
        return Domain.from_dict(data)

    def list(self) -> List[Domain]:
        """
        List all domains.

        Returns:
            List of domains
        """
        data = self._http.get("/v1/domains")
        # API returns array directly
        if isinstance(data, list):
            return [Domain.from_dict(d) for d in data]
        # Or wrapped in data/pagination
        return [Domain.from_dict(d) for d in data.get("data", data)]


    def update(
        self,
        id: str,
        *,
        receiving_enabled: Optional[bool] = None,
    ) -> Domain:
        """
        Update domain settings.

        Args:
            id: Domain ID
            receiving_enabled: Whether to enable receiving emails

        Returns:
            Updated domain
        """
        request = UpdateDomainRequest(receiving_enabled=receiving_enabled)
        data = self._http.put(f"/v1/domains/{id}", request.to_dict())
        return Domain.from_dict(data)

    def delete(self, id: str) -> SuccessResponse:
        """
        Delete a domain.

        Args:
            id: Domain ID

        Returns:
            Success response
        """
        data = self._http.delete(f"/v1/domains/{id}")
        return SuccessResponse(success=data.get("success", True))

    def verify(self, id: str) -> DomainVerificationStatus:
        """
        Verify domain DNS records.

        Args:
            id: Domain ID

        Returns:
            Domain verification status
        """
        data = self._http.post(f"/v1/domains/{id}/verify")
        return DomainVerificationStatus.from_dict(data)

    def check_dns(self, id: str) -> DnsVerificationStatus:
        """
        Check DNS verification status.

        Args:
            id: Domain ID

        Returns:
            DNS verification status for all records
        """
        data = self._http.get(f"/v1/domains/{id}/dns-status")
        return DnsVerificationStatus.from_dict(data)


class AsyncDomainsResource:
    """Asynchronous domain resource for managing email domains."""

    def __init__(self, http: AsyncHttpClient) -> None:
        self._http = http

    async def add(
        self,
        domain: str,
        *,
        receiving_subdomain: Optional[str] = None,
    ) -> Domain:
        """Add a new domain."""
        request = AddDomainRequest(domain=domain, receiving_subdomain=receiving_subdomain)
        data = await self._http.post("/v1/domains", request.to_dict())
        return Domain.from_dict(data)

    async def get(self, id: str) -> Domain:
        """Get domain by ID."""
        data = await self._http.get(f"/v1/domains/{id}")
        return Domain.from_dict(data)

    async def list(self) -> List[Domain]:
        """List all domains."""
        data = await self._http.get("/v1/domains")
        # API returns array directly
        if isinstance(data, list):
            return [Domain.from_dict(d) for d in data]
        # Or wrapped in data/pagination
        return [Domain.from_dict(d) for d in data.get("data", data)]

    async def update(
        self,
        id: str,
        *,
        receiving_enabled: Optional[bool] = None,
    ) -> Domain:
        """Update domain settings."""
        request = UpdateDomainRequest(receiving_enabled=receiving_enabled)
        data = await self._http.put(f"/v1/domains/{id}", request.to_dict())
        return Domain.from_dict(data)

    async def delete(self, id: str) -> SuccessResponse:
        """Delete a domain."""
        data = await self._http.delete(f"/v1/domains/{id}")
        return SuccessResponse(success=data.get("success", True))

    async def verify(self, id: str) -> DomainVerificationStatus:
        """Verify domain DNS records."""
        data = await self._http.post(f"/v1/domains/{id}/verify")
        return DomainVerificationStatus.from_dict(data)

    async def check_dns(self, id: str) -> DnsVerificationStatus:
        """Check DNS verification status."""
        data = await self._http.get(f"/v1/domains/{id}/dns-status")
        return DnsVerificationStatus.from_dict(data)



"""
Tests for Emailr SDK error classes.
"""

import pytest
from emailr.errors import (
    EmailrError,
    NetworkError,
    ValidationError,
    AuthenticationError,
    NotFoundError,
    RateLimitError,
)


class TestEmailrError:
    """Tests for the base EmailrError class."""

    def test_create_with_message(self):
        """Should create an error with message."""
        error = EmailrError("Something went wrong")
        
        assert str(error) == "Something went wrong"
        assert error.message == "Something went wrong"
        assert error.status_code is None

    def test_create_with_status_code(self):
        """Should create an error with status code."""
        error = EmailrError("Server error", status_code=500)
        
        assert error.message == "Server error"
        assert error.status_code == 500

    def test_create_with_code_and_details(self):
        """Should create an error with code and details."""
        error = EmailrError(
            "Bad request",
            status_code=400,
            code="INVALID_INPUT",
            details={"field": "email"}
        )
        
        assert error.code == "INVALID_INPUT"
        assert error.details == {"field": "email"}

    def test_is_exception(self):
        """Should be catchable as Exception."""
        error = EmailrError("Test error")
        
        assert isinstance(error, Exception)
        
        with pytest.raises(Exception):
            raise error


class TestNetworkError:
    """Tests for NetworkError class."""

    def test_create_network_error(self):
        """Should create a network error."""
        error = NetworkError("Connection failed")
        
        assert error.message == "Connection failed"
        assert isinstance(error, EmailrError)

    def test_with_original_error(self):
        """Should include original error."""
        original = ConnectionError("ECONNREFUSED")
        error = NetworkError("Connection failed", original_error=original)
        
        assert error.original_error is original


class TestValidationError:
    """Tests for ValidationError class."""

    def test_create_validation_error(self):
        """Should create a validation error with details."""
        details = {
            "email": ["Invalid email format"],
            "subject": ["Subject is required"],
        }
        error = ValidationError("Validation failed", details=details)
        
        assert error.message == "Validation failed"
        assert error.status_code == 400
        assert error.details == details

    def test_is_emailr_error(self):
        """Should be an EmailrError."""
        error = ValidationError("Invalid input")
        
        assert isinstance(error, EmailrError)


class TestAuthenticationError:
    """Tests for AuthenticationError class."""

    def test_create_auth_error(self):
        """Should create an authentication error."""
        error = AuthenticationError("Invalid API key")
        
        assert error.message == "Invalid API key"
        assert error.status_code == 401

    def test_default_message(self):
        """Should use default message."""
        error = AuthenticationError()
        
        assert error.message == "Authentication failed"


class TestNotFoundError:
    """Tests for NotFoundError class."""

    def test_create_not_found_error(self):
        """Should create a not found error."""
        error = NotFoundError("Email not found")
        
        assert error.message == "Email not found"
        assert error.status_code == 404

    def test_default_message(self):
        """Should use default message."""
        error = NotFoundError()
        
        assert error.message == "Resource not found"


class TestRateLimitError:
    """Tests for RateLimitError class."""

    def test_create_rate_limit_error(self):
        """Should create a rate limit error."""
        error = RateLimitError("Too many requests", retry_after=60)
        
        assert error.message == "Too many requests"
        assert error.status_code == 429
        assert error.retry_after == 60

    def test_default_message(self):
        """Should use default message."""
        error = RateLimitError()
        
        assert error.message == "Rate limit exceeded"
        assert error.retry_after is None

    def test_retry_after_optional(self):
        """Should allow retry_after to be optional."""
        error = RateLimitError("Rate limited")
        
        assert error.retry_after is None


class TestErrorHierarchy:
    """Tests for error class hierarchy."""

    def test_all_errors_inherit_from_emailr_error(self):
        """All error types should inherit from EmailrError."""
        errors = [
            NetworkError("test"),
            ValidationError("test"),
            AuthenticationError("test"),
            NotFoundError("test"),
            RateLimitError("test"),
        ]
        
        for error in errors:
            assert isinstance(error, EmailrError)

    def test_catch_specific_error(self):
        """Should be able to catch specific error types."""
        with pytest.raises(ValidationError):
            raise ValidationError("Invalid input")

    def test_catch_base_error(self):
        """Should be able to catch all errors with base class."""
        with pytest.raises(EmailrError):
            raise NotFoundError("Not found")



"""
API Keys resource for managing API keys.
"""

from typing import Any, Dict, List

from emailr.http import AsyncHttpClient, HttpClient
from emailr.types import (
    ApiKey,
    ApiKeyListItem,
    CreateApiKeyRequest,
    SuccessResponse,
)


class ApiKeysResource:
    """Synchronous API keys resource."""

    def __init__(self, http: HttpClient) -> None:
        self._http = http

    def create(
        self,
        name: str,
        *,
        permissions: List[str] | None = None,
    ) -> ApiKey:
        """
        Create a new API key.
        Note: The full key is only returned once upon creation.

        Args:
            name: Name for the API key
            permissions: Optional list of permissions

        Returns:
            Created API key with the full key value
        """
        request = CreateApiKeyRequest(name=name, permissions=permissions)
        data = self._http.post("/v1/api-keys", request.to_dict())
        return ApiKey.from_dict(data)

    def list(self) -> List[ApiKeyListItem]:
        """
        List all active API keys.
        Note: Full keys are not returned for security.

        Returns:
            List of API key metadata
        """
        data = self._http.get("/v1/api-keys")
        if isinstance(data, list):
            return [ApiKeyListItem.from_dict(k) for k in data]
        return [ApiKeyListItem.from_dict(k) for k in data.get("data", data)]

    def revoke(self, id: str) -> SuccessResponse:
        """
        Revoke an API key.

        Args:
            id: API key ID

        Returns:
            Success response
        """
        data = self._http.delete(f"/v1/api-keys/{id}")
        return SuccessResponse(success=data.get("success", True))


class AsyncApiKeysResource:
    """Asynchronous API keys resource."""

    def __init__(self, http: AsyncHttpClient) -> None:
        self._http = http

    async def create(
        self,
        name: str,
        *,
        permissions: List[str] | None = None,
    ) -> ApiKey:
        """Create a new API key."""
        request = CreateApiKeyRequest(name=name, permissions=permissions)
        data = await self._http.post("/v1/api-keys", request.to_dict())
        return ApiKey.from_dict(data)

    async def list(self) -> List[ApiKeyListItem]:
        """List all active API keys."""
        data = await self._http.get("/v1/api-keys")
        if isinstance(data, list):
            return [ApiKeyListItem.from_dict(k) for k in data]
        return [ApiKeyListItem.from_dict(k) for k in data.get("data", data)]

    async def revoke(self, id: str) -> SuccessResponse:
        """Revoke an API key."""
        data = await self._http.delete(f"/v1/api-keys/{id}")
        return SuccessResponse(success=data.get("success", True))



"""
Main client classes for the Emailr SDK.

Provides both synchronous (Emailr) and asynchronous (AsyncEmailr) clients
for interacting with the Emailr API.
"""

from types import TracebackType
from typing import Optional, Type

from emailr.http import AsyncHttpClient, HttpClient
from emailr.resources import (
    AsyncBroadcastsResource,
    AsyncContactsResource,
    AsyncDomainsResource,
    AsyncEmailsResource,
    AsyncSegmentsResource,
    AsyncTemplatesResource,
    AsyncWebhooksResource,
    AsyncApiKeysResource,
    AsyncSmtpResource,
    AsyncSettingsResource,
    AsyncMetricsResource,
    BroadcastsResource,
    ContactsResource,
    DomainsResource,
    EmailsResource,
    SegmentsResource,
    TemplatesResource,
    WebhooksResource,
    ApiKeysResource,
    SmtpResource,
    SettingsResource,
    MetricsResource,
)

DEFAULT_BASE_URL = "https://api.emailr.dev"
DEFAULT_TIMEOUT = 30.0


class Emailr:
    """
    Synchronous Emailr API client.

    Example usage:
        >>> from emailr import Emailr
        >>> client = Emailr(api_key="your-api-key")
        >>> result = client.emails.send(
        ...     to="recipient@example.com",
        ...     from_="sender@yourdomain.com",
        ...     subject="Hello!",
        ...     html="<h1>Hello World</h1>"
        ... )
        >>> print(result.message_id)

    Args:
        api_key: Your Emailr API key
        base_url: Base URL for the API (default: https://api.emailr.dev)
        timeout: Request timeout in seconds (default: 30.0)
    """

    def __init__(
        self,
        api_key: str,
        *,
        base_url: str = DEFAULT_BASE_URL,
        timeout: float = DEFAULT_TIMEOUT,
    ) -> None:
        if not api_key:
            raise ValueError("API key is required")

        self._http = HttpClient(api_key=api_key, base_url=base_url, timeout=timeout)

        # Initialize resources
        self.emails = EmailsResource(self._http)
        self.contacts = ContactsResource(self._http)
        self.templates = TemplatesResource(self._http)
        self.domains = DomainsResource(self._http)
        self.webhooks = WebhooksResource(self._http)
        self.broadcasts = BroadcastsResource(self._http)
        self.segments = SegmentsResource(self._http)
        self.api_keys = ApiKeysResource(self._http)
        self.smtp = SmtpResource(self._http)
        self.settings = SettingsResource(self._http)
        self.metrics = MetricsResource(self._http)


    def close(self) -> None:
        """Close the HTTP client and release resources."""
        self._http.close()

    def __enter__(self) -> "Emailr":
        """Enter context manager."""
        return self

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        """Exit context manager and close client."""
        self.close()


class AsyncEmailr:
    """
    Asynchronous Emailr API client.

    Example usage:
        >>> from emailr import AsyncEmailr
        >>> async with AsyncEmailr(api_key="your-api-key") as client:
        ...     result = await client.emails.send(
        ...         to="recipient@example.com",
        ...         from_="sender@yourdomain.com",
        ...         subject="Hello!",
        ...         html="<h1>Hello World</h1>"
        ...     )
        ...     print(result.message_id)

    Args:
        api_key: Your Emailr API key
        base_url: Base URL for the API (default: https://api.emailr.dev)
        timeout: Request timeout in seconds (default: 30.0)
    """

    def __init__(
        self,
        api_key: str,
        *,
        base_url: str = DEFAULT_BASE_URL,
        timeout: float = DEFAULT_TIMEOUT,
    ) -> None:
        if not api_key:
            raise ValueError("API key is required")

        self._http = AsyncHttpClient(api_key=api_key, base_url=base_url, timeout=timeout)

        # Initialize resources
        self.emails = AsyncEmailsResource(self._http)
        self.contacts = AsyncContactsResource(self._http)
        self.templates = AsyncTemplatesResource(self._http)
        self.domains = AsyncDomainsResource(self._http)
        self.webhooks = AsyncWebhooksResource(self._http)
        self.broadcasts = AsyncBroadcastsResource(self._http)
        self.segments = AsyncSegmentsResource(self._http)
        self.api_keys = AsyncApiKeysResource(self._http)
        self.smtp = AsyncSmtpResource(self._http)
        self.settings = AsyncSettingsResource(self._http)
        self.metrics = AsyncMetricsResource(self._http)

    async def close(self) -> None:
        """Close the HTTP client and release resources."""
        await self._http.close()

    async def __aenter__(self) -> "AsyncEmailr":
        """Enter async context manager."""
        return self

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        """Exit async context manager and close client."""
        await self.close()



"""
Exception classes for the Emailr SDK.

All exceptions inherit from EmailrError, which provides common attributes
for error handling including status code, error code, and request ID.
"""

from typing import Any, Dict, Optional


class EmailrError(Exception):
    """
    Base exception class for all Emailr SDK errors.

    Attributes:
        message: Human-readable error message
        status_code: HTTP status code (0 for network errors)
        code: Error code string for programmatic handling
        request_id: Request ID for debugging (when available)
    """

    def __init__(
        self,
        message: str,
        status_code: int,
        code: Optional[str] = None,
        request_id: Optional[str] = None,
    ) -> None:
        super().__init__(message)
        self.message = message
        self.status_code = status_code
        self.code = code
        self.request_id = request_id

    def __str__(self) -> str:
        parts = [self.message]
        if self.code:
            parts.append(f"[{self.code}]")
        if self.request_id:
            parts.append(f"(request_id: {self.request_id})")
        return " ".join(parts)

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}("
            f"message={self.message!r}, "
            f"status_code={self.status_code}, "
            f"code={self.code!r}, "
            f"request_id={self.request_id!r})"
        )


class NetworkError(EmailrError):
    """
    Exception raised when a network request fails.

    This includes connection errors, timeouts, and other transport-level failures.
    """

    def __init__(self, message: str, cause: Optional[Exception] = None) -> None:
        super().__init__(message, status_code=0, code="NETWORK_ERROR")
        self.__cause__ = cause


class AuthenticationError(EmailrError):
    """
    Exception raised when authentication fails.

    This typically indicates an invalid or expired API key.
    """

    def __init__(
        self,
        message: str = "Invalid API key",
        request_id: Optional[str] = None,
    ) -> None:
        super().__init__(
            message,
            status_code=401,
            code="AUTHENTICATION_ERROR",
            request_id=request_id,
        )


class RateLimitError(EmailrError):
    """
    Exception raised when the rate limit is exceeded.

    Attributes:
        retry_after: Number of seconds to wait before retrying (when available)
    """

    def __init__(
        self,
        message: str = "Rate limit exceeded",
        retry_after: Optional[int] = None,
        request_id: Optional[str] = None,
    ) -> None:
        super().__init__(
            message,
            status_code=429,
            code="RATE_LIMIT_ERROR",
            request_id=request_id,
        )
        self.retry_after = retry_after


class NotFoundError(EmailrError):
    """
    Exception raised when a requested resource is not found.
    """

    def __init__(
        self,
        message: str = "Resource not found",
        request_id: Optional[str] = None,
    ) -> None:
        super().__init__(
            message,
            status_code=404,
            code="NOT_FOUND",
            request_id=request_id,
        )


class ValidationError(EmailrError):
    """
    Exception raised when request validation fails.

    Attributes:
        details: Additional details about validation errors (when available)
    """

    def __init__(
        self,
        message: str,
        details: Optional[Dict[str, Any]] = None,
        request_id: Optional[str] = None,
    ) -> None:
        super().__init__(
            message,
            status_code=400,
            code="VALIDATION_ERROR",
            request_id=request_id,
        )
        self.details = details



"""
Type definitions for the Emailr SDK.

All types are implemented as dataclasses with full type hints.
These types correspond to the API request and response schemas.
"""

from dataclasses import dataclass, field
from typing import Any, Dict, Generic, List, Optional, TypeVar, Union

T = TypeVar("T")


# ============================================================================
# Common Types
# ============================================================================


@dataclass
class Pagination:
    """Pagination information for list responses."""

    page: int
    limit: int
    total: int
    pages: int


@dataclass
class PaginatedResponse(Generic[T]):
    """Generic paginated response wrapper."""

    data: List[T]
    pagination: Pagination


@dataclass
class SuccessResponse:
    """Simple success response."""

    success: bool


# ============================================================================
# Email Types
# ============================================================================


@dataclass
class Attachment:
    """Email attachment."""

    filename: str
    content: str  # Base64 encoded
    content_type: Optional[str] = None


@dataclass
class ReplyTo:
    """Reply-to configuration for threading."""

    in_reply_to: Optional[str] = None
    thread_id: Optional[str] = None
    parent_email_id: Optional[str] = None


@dataclass
class SendEmailRequest:
    """Request payload for sending an email."""

    to: Union[str, List[str]]
    from_: Optional[str] = None
    cc: Optional[Union[str, List[str]]] = None
    bcc: Optional[Union[str, List[str]]] = None
    subject: Optional[str] = None
    html: Optional[str] = None
    text: Optional[str] = None
    template_id: Optional[str] = None
    template_data: Optional[Dict[str, Any]] = None
    tags: Optional[Dict[str, str]] = None
    attachments: Optional[List[Attachment]] = None
    reply_to: Optional[ReplyTo] = None
    reply_to_email: Optional[str] = None
    preview_text: Optional[str] = None
    scheduled_at: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {"to": self.to}
        if self.from_ is not None:
            data["from"] = self.from_
        if self.cc is not None:
            data["cc"] = self.cc
        if self.bcc is not None:
            data["bcc"] = self.bcc
        if self.subject is not None:
            data["subject"] = self.subject
        if self.html is not None:
            data["html"] = self.html
        if self.text is not None:
            data["text"] = self.text
        if self.template_id is not None:
            data["template_id"] = self.template_id
        if self.template_data is not None:
            data["template_data"] = self.template_data
        if self.tags is not None:
            data["tags"] = self.tags
        if self.attachments is not None:
            data["attachments"] = [
                {
                    "filename": a.filename,
                    "content": a.content,
                    "contentType": a.content_type,
                }
                for a in self.attachments
            ]
        if self.reply_to is not None:
            data["replyTo"] = {
                "in_reply_to": self.reply_to.in_reply_to,
                "thread_id": self.reply_to.thread_id,
                "parent_email_id": self.reply_to.parent_email_id,
            }
        if self.reply_to_email is not None:
            data["reply_to_email"] = self.reply_to_email
        if self.preview_text is not None:
            data["preview_text"] = self.preview_text
        if self.scheduled_at is not None:
            data["scheduled_at"] = self.scheduled_at
        return data


@dataclass
class SendEmailResponse:
    """Response from sending an email."""

    success: bool
    message_id: str
    recipients: int
    status: str
    scheduled_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "SendEmailResponse":
        """Create from API response dictionary."""
        return cls(
            success=data["success"],
            message_id=data["message_id"],
            recipients=data["recipients"],
            status=data["status"],
            scheduled_at=data.get("scheduled_at"),
        )


@dataclass
class Email:
    """Email record."""

    id: str
    organization_id: str
    message_id: str
    from_email: str
    to_email: str
    subject: Optional[str]
    html_content: Optional[str]
    text_content: Optional[str]
    template_id: Optional[str]
    status: str
    ses_message_id: Optional[str]
    broadcast_id: Optional[str]
    metadata: Optional[Dict[str, Any]]
    sent_at: Optional[str]
    delivered_at: Optional[str]
    opened_at: Optional[str]
    clicked_at: Optional[str]
    bounced_at: Optional[str]
    complained_at: Optional[str]
    scheduled_at: Optional[str]
    created_at: str
    thread_id: Optional[str]
    parent_email_id: Optional[str]
    attachments: Optional[List[Any]]
    clicked_links: Optional[List[Any]]
    opens: Optional[List[Any]]

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Email":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            organization_id=data["organization_id"],
            message_id=data["message_id"],
            from_email=data["from_email"],
            to_email=data["to_email"],
            subject=data.get("subject"),
            html_content=data.get("html_content"),
            text_content=data.get("text_content"),
            template_id=data.get("template_id"),
            status=data["status"],
            ses_message_id=data.get("ses_message_id"),
            broadcast_id=data.get("broadcast_id"),
            metadata=data.get("metadata"),
            sent_at=data.get("sent_at"),
            delivered_at=data.get("delivered_at"),
            opened_at=data.get("opened_at"),
            clicked_at=data.get("clicked_at"),
            bounced_at=data.get("bounced_at"),
            complained_at=data.get("complained_at"),
            scheduled_at=data.get("scheduled_at"),
            created_at=data["created_at"],
            thread_id=data.get("thread_id"),
            parent_email_id=data.get("parent_email_id"),
            attachments=data.get("attachments"),
            clicked_links=data.get("clicked_links"),
            opens=data.get("opens"),
        )


@dataclass
class ListEmailsParams:
    """Parameters for listing emails."""

    page: Optional[int] = None
    limit: Optional[int] = None


@dataclass
class ForwardEmailRequest:
    """Request payload for forwarding an email."""

    email_id: str
    to: Union[str, List[str]]
    message: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {"email_id": self.email_id, "to": self.to}
        if self.message is not None:
            data["message"] = self.message
        return data


@dataclass
class ForwardingRule:
    """Email forwarding rule."""

    id: str
    name: str
    from_pattern: str
    to_addresses: List[str]
    active: bool
    created_at: str

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ForwardingRule":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            name=data["name"],
            from_pattern=data["from_pattern"],
            to_addresses=data["to_addresses"],
            active=data["active"],
            created_at=data["created_at"],
        )


@dataclass
class CreateForwardingRuleRequest:
    """Request payload for creating a forwarding rule."""

    name: str
    from_pattern: str
    to_addresses: List[str]
    active: Optional[bool] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {
            "name": self.name,
            "from_pattern": self.from_pattern,
            "to_addresses": self.to_addresses,
        }
        if self.active is not None:
            data["active"] = self.active
        return data


# ============================================================================
# Contact Types
# ============================================================================


@dataclass
class Contact:
    """Contact record."""

    id: str
    organization_id: str
    email: str
    first_name: Optional[str]
    last_name: Optional[str]
    metadata: Optional[Dict[str, Any]]
    subscribed: bool
    unsubscribed_at: Optional[str]
    created_at: str
    updated_at: str

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Contact":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            organization_id=data["organization_id"],
            email=data["email"],
            first_name=data.get("first_name"),
            last_name=data.get("last_name"),
            metadata=data.get("metadata"),
            subscribed=data["subscribed"],
            unsubscribed_at=data.get("unsubscribed_at"),
            created_at=data["created_at"],
            updated_at=data["updated_at"],
        )


@dataclass
class CreateContactRequest:
    """Request payload for creating a contact."""

    email: str
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    subscribed: Optional[bool] = None
    metadata: Optional[Dict[str, Any]] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {"email": self.email}
        if self.first_name is not None:
            data["first_name"] = self.first_name
        if self.last_name is not None:
            data["last_name"] = self.last_name
        if self.subscribed is not None:
            data["subscribed"] = self.subscribed
        if self.metadata is not None:
            data["metadata"] = self.metadata
        return data


@dataclass
class UpdateContactRequest:
    """Request payload for updating a contact."""

    email: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    subscribed: Optional[bool] = None
    metadata: Optional[Dict[str, Any]] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {}
        if self.email is not None:
            data["email"] = self.email
        if self.first_name is not None:
            data["first_name"] = self.first_name
        if self.last_name is not None:
            data["last_name"] = self.last_name
        if self.subscribed is not None:
            data["subscribed"] = self.subscribed
        if self.metadata is not None:
            data["metadata"] = self.metadata
        return data


@dataclass
class ListContactsParams:
    """Parameters for listing contacts."""

    limit: Optional[int] = None
    offset: Optional[int] = None
    subscribed: Optional[bool] = None


@dataclass
class ContactListResponse:
    """Response from listing contacts."""

    contacts: List[Contact]
    total: int
    limit: int
    offset: int

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ContactListResponse":
        """Create from API response dictionary."""
        return cls(
            contacts=[Contact.from_dict(c) for c in data["contacts"]],
            total=data["total"],
            limit=data["limit"],
            offset=data["offset"],
        )


@dataclass
class BulkCreateContactsRequest:
    """Request payload for bulk creating contacts."""

    contacts: List[CreateContactRequest]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        return {"contacts": [c.to_dict() for c in self.contacts]}


@dataclass
class BulkCreateContactsResponse:
    """Response from bulk creating contacts."""

    imported: int
    total: int

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "BulkCreateContactsResponse":
        """Create from API response dictionary."""
        return cls(imported=data["imported"], total=data["total"])


# ============================================================================
# Template Types
# ============================================================================


@dataclass
class Template:
    """Email template record."""

    id: str
    organization_id: str
    name: str
    subject: str
    html_content: Optional[str]
    text_content: Optional[str]
    variables: Optional[List[str]]
    from_email: Optional[str]
    reply_to: Optional[str]
    preview_text: Optional[str]
    created_by: Optional[str]
    created_at: str
    updated_at: str

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Template":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            organization_id=data["organization_id"],
            name=data["name"],
            subject=data["subject"],
            html_content=data.get("html_content"),
            text_content=data.get("text_content"),
            variables=data.get("variables"),
            from_email=data.get("from_email"),
            reply_to=data.get("reply_to"),
            preview_text=data.get("preview_text"),
            created_by=data.get("created_by"),
            created_at=data["created_at"],
            updated_at=data["updated_at"],
        )


@dataclass
class CreateTemplateRequest:
    """Request payload for creating a template."""

    name: str
    subject: str
    html_content: Optional[str] = None
    text_content: Optional[str] = None
    variables: Optional[List[str]] = None
    from_email: Optional[str] = None
    reply_to: Optional[str] = None
    preview_text: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {"name": self.name, "subject": self.subject}
        if self.html_content is not None:
            data["html_content"] = self.html_content
        if self.text_content is not None:
            data["text_content"] = self.text_content
        if self.variables is not None:
            data["variables"] = self.variables
        if self.from_email is not None:
            data["from_email"] = self.from_email
        if self.reply_to is not None:
            data["reply_to"] = self.reply_to
        if self.preview_text is not None:
            data["preview_text"] = self.preview_text
        return data


@dataclass
class UpdateTemplateRequest:
    """Request payload for updating a template."""

    name: Optional[str] = None
    subject: Optional[str] = None
    html_content: Optional[str] = None
    text_content: Optional[str] = None
    variables: Optional[List[str]] = None
    from_email: Optional[str] = None
    reply_to: Optional[str] = None
    preview_text: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {}
        if self.name is not None:
            data["name"] = self.name
        if self.subject is not None:
            data["subject"] = self.subject
        if self.html_content is not None:
            data["html_content"] = self.html_content
        if self.text_content is not None:
            data["text_content"] = self.text_content
        if self.variables is not None:
            data["variables"] = self.variables
        if self.from_email is not None:
            data["from_email"] = self.from_email
        if self.reply_to is not None:
            data["reply_to"] = self.reply_to
        if self.preview_text is not None:
            data["preview_text"] = self.preview_text
        return data


@dataclass
class ListTemplatesParams:
    """Parameters for listing templates."""

    page: Optional[int] = None
    limit: Optional[int] = None


# ============================================================================
# Domain Types
# ============================================================================


@dataclass
class DnsRecord:
    """DNS record for domain verification."""

    type: str  # 'TXT' | 'MX' | 'CNAME'
    name: str
    value: str
    priority: Optional[int] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "DnsRecord":
        """Create from API response dictionary."""
        return cls(
            type=data["type"],
            name=data["name"],
            value=data["value"],
            priority=data.get("priority"),
        )


@dataclass
class DomainDnsRecords:
    """DNS records for domain configuration."""

    dkim: DnsRecord
    spf: DnsRecord
    dmarc: DnsRecord
    mail_from_mx: DnsRecord
    mail_from_spf: DnsRecord
    receiving_mx: DnsRecord

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "DomainDnsRecords":
        """Create from API response dictionary."""
        return cls(
            dkim=DnsRecord.from_dict(data["dkim"]),
            spf=DnsRecord.from_dict(data["spf"]),
            dmarc=DnsRecord.from_dict(data["dmarc"]),
            mail_from_mx=DnsRecord.from_dict(data["mailFromMx"]),
            mail_from_spf=DnsRecord.from_dict(data["mailFromSpf"]),
            receiving_mx=DnsRecord.from_dict(data["receivingMx"]),
        )


@dataclass
class Domain:
    """Domain record."""

    id: str
    organization_id: str
    domain: str
    status: str
    verification_token: Optional[str]
    dkim_verified: bool
    spf_verified: bool
    dmarc_verified: bool
    cloudflare_zone_id: Optional[str]
    ses_identity_arn: Optional[str]
    created_at: str
    verified_at: Optional[str]
    dkim_tokens: Optional[List[str]]
    dns_records: Optional[DomainDnsRecords]
    dkim_selector: Optional[str]
    mail_from_subdomain: Optional[str]
    mail_from_verified: Optional[bool]
    receiving_enabled: Optional[bool]
    receiving_subdomain: Optional[str]

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Domain":
        """Create from API response dictionary."""
        dns_records = None
        if data.get("dns_records"):
            dns_records = DomainDnsRecords.from_dict(data["dns_records"])
        return cls(
            id=data["id"],
            organization_id=data["organization_id"],
            domain=data["domain"],
            status=data["status"],
            verification_token=data.get("verification_token"),
            dkim_verified=data["dkim_verified"],
            spf_verified=data["spf_verified"],
            dmarc_verified=data["dmarc_verified"],
            cloudflare_zone_id=data.get("cloudflare_zone_id"),
            ses_identity_arn=data.get("ses_identity_arn"),
            created_at=data["created_at"],
            verified_at=data.get("verified_at"),
            dkim_tokens=data.get("dkim_tokens"),
            dns_records=dns_records,
            dkim_selector=data.get("dkim_selector"),
            mail_from_subdomain=data.get("mail_from_subdomain"),
            mail_from_verified=data.get("mail_from_verified"),
            receiving_enabled=data.get("receiving_enabled"),
            receiving_subdomain=data.get("receiving_subdomain"),
        )


@dataclass
class AddDomainRequest:
    """Request payload for adding a domain."""

    domain: str
    receiving_subdomain: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {"domain": self.domain}
        if self.receiving_subdomain is not None:
            data["receivingSubdomain"] = self.receiving_subdomain
        return data


@dataclass
class UpdateDomainRequest:
    """Request payload for updating a domain."""

    receiving_enabled: Optional[bool] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {}
        if self.receiving_enabled is not None:
            data["receiving_enabled"] = self.receiving_enabled
        return data


@dataclass
class DomainVerificationStatus:
    """Domain verification status."""

    verified: bool
    status: str
    dkim_status: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "DomainVerificationStatus":
        """Create from API response dictionary."""
        return cls(
            verified=data["verified"],
            status=data["status"],
            dkim_status=data.get("dkimStatus"),
        )


@dataclass
class DnsVerificationStatus:
    """DNS verification status for all records."""

    records: Dict[str, Dict[str, Any]]

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "DnsVerificationStatus":
        """Create from API response dictionary."""
        return cls(records=data)


# ============================================================================
# Webhook Types
# ============================================================================


@dataclass
class Webhook:
    """Webhook record."""

    id: str
    organization_id: str
    name: str
    url: str
    events: List[str]
    secret: str
    active: bool
    created_by: Optional[str]
    created_at: str

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Webhook":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            organization_id=data["organization_id"],
            name=data["name"],
            url=data["url"],
            events=data["events"],
            secret=data["secret"],
            active=data["active"],
            created_by=data.get("created_by"),
            created_at=data["created_at"],
        )


@dataclass
class CreateWebhookRequest:
    """Request payload for creating a webhook."""

    name: str
    url: str
    events: List[str]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        return {"name": self.name, "url": self.url, "events": self.events}


@dataclass
class UpdateWebhookRequest:
    """Request payload for updating a webhook."""

    name: Optional[str] = None
    url: Optional[str] = None
    events: Optional[List[str]] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {}
        if self.name is not None:
            data["name"] = self.name
        if self.url is not None:
            data["url"] = self.url
        if self.events is not None:
            data["events"] = self.events
        return data


@dataclass
class WebhookToggleResponse:
    """Response from enabling/disabling a webhook."""

    success: bool
    active: bool

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "WebhookToggleResponse":
        """Create from API response dictionary."""
        return cls(success=data["success"], active=data["active"])


@dataclass
class WebhookDelivery:
    """Webhook delivery record."""

    id: str
    webhook_id: str
    event_type: str
    payload: Dict[str, Any]
    response_status: Optional[int]
    response_body: Optional[str]
    attempt_count: int
    delivered_at: Optional[str]
    created_at: str

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "WebhookDelivery":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            webhook_id=data["webhook_id"],
            event_type=data["event_type"],
            payload=data["payload"],
            response_status=data.get("response_status"),
            response_body=data.get("response_body"),
            attempt_count=data["attempt_count"],
            delivered_at=data.get("delivered_at"),
            created_at=data["created_at"],
        )


@dataclass
class ListWebhooksParams:
    """Parameters for listing webhooks."""

    page: Optional[int] = None
    limit: Optional[int] = None


# ============================================================================
# Broadcast Types
# ============================================================================


@dataclass
class Broadcast:
    """Broadcast campaign record."""

    id: str
    organization_id: str
    name: str
    subject: str
    from_email: str
    template_id: Optional[str]
    segment_id: Optional[str]
    status: str
    total_recipients: Optional[int]
    sent_count: Optional[int]
    delivered_count: Optional[int]
    opened_count: Optional[int]
    clicked_count: Optional[int]
    bounced_count: Optional[int]
    scheduled_at: Optional[str]
    started_at: Optional[str]
    completed_at: Optional[str]
    created_by: Optional[str]
    created_at: str

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Broadcast":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            organization_id=data["organization_id"],
            name=data["name"],
            subject=data["subject"],
            from_email=data["from_email"],
            template_id=data.get("template_id"),
            segment_id=data.get("segment_id"),
            status=data["status"],
            total_recipients=data.get("total_recipients"),
            sent_count=data.get("sent_count"),
            delivered_count=data.get("delivered_count"),
            opened_count=data.get("opened_count"),
            clicked_count=data.get("clicked_count"),
            bounced_count=data.get("bounced_count"),
            scheduled_at=data.get("scheduled_at"),
            started_at=data.get("started_at"),
            completed_at=data.get("completed_at"),
            created_by=data.get("created_by"),
            created_at=data["created_at"],
        )


@dataclass
class CreateBroadcastRequest:
    """Request payload for creating a broadcast."""

    name: str
    subject: str
    from_email: str
    template_id: Optional[str] = None
    segment_id: Optional[str] = None
    html_content: Optional[str] = None
    text_content: Optional[str] = None
    scheduled_at: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {
            "name": self.name,
            "subject": self.subject,
            "from_email": self.from_email,
        }
        if self.template_id is not None:
            data["template_id"] = self.template_id
        if self.segment_id is not None:
            data["segment_id"] = self.segment_id
        if self.html_content is not None:
            data["html_content"] = self.html_content
        if self.text_content is not None:
            data["text_content"] = self.text_content
        if self.scheduled_at is not None:
            data["scheduled_at"] = self.scheduled_at
        return data


@dataclass
class UpdateBroadcastRequest:
    """Request payload for updating a broadcast."""

    name: Optional[str] = None
    subject: Optional[str] = None
    from_email: Optional[str] = None
    template_id: Optional[str] = None
    segment_id: Optional[str] = None
    html_content: Optional[str] = None
    text_content: Optional[str] = None
    scheduled_at: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {}
        if self.name is not None:
            data["name"] = self.name
        if self.subject is not None:
            data["subject"] = self.subject
        if self.from_email is not None:
            data["from_email"] = self.from_email
        if self.template_id is not None:
            data["template_id"] = self.template_id
        if self.segment_id is not None:
            data["segment_id"] = self.segment_id
        if self.html_content is not None:
            data["html_content"] = self.html_content
        if self.text_content is not None:
            data["text_content"] = self.text_content
        if self.scheduled_at is not None:
            data["scheduled_at"] = self.scheduled_at
        return data


@dataclass
class SendBroadcastResponse:
    """Response from sending a broadcast."""

    success: bool
    sent: int
    total: int

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "SendBroadcastResponse":
        """Create from API response dictionary."""
        return cls(success=data["success"], sent=data["sent"], total=data["total"])


@dataclass
class ListBroadcastsParams:
    """Parameters for listing broadcasts."""

    page: Optional[int] = None
    limit: Optional[int] = None
    status: Optional[str] = None


@dataclass
class BroadcastStats:
    """Broadcast statistics."""

    total_recipients: int
    sent_count: int
    delivered_count: int
    opened_count: int
    clicked_count: int
    bounced_count: int

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "BroadcastStats":
        """Create from API response dictionary."""
        return cls(
            total_recipients=data["total_recipients"],
            sent_count=data["sent_count"],
            delivered_count=data["delivered_count"],
            opened_count=data["opened_count"],
            clicked_count=data["clicked_count"],
            bounced_count=data["bounced_count"],
        )


# ============================================================================
# Segment Types
# ============================================================================


@dataclass
class Segment:
    """Contact segment record."""

    id: str
    organization_id: str
    name: str
    description: Optional[str]
    conditions: Dict[str, Any]
    created_by: Optional[str]
    created_at: str
    updated_at: str

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Segment":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            organization_id=data["organization_id"],
            name=data["name"],
            description=data.get("description"),
            conditions=data["conditions"],
            created_by=data.get("created_by"),
            created_at=data["created_at"],
            updated_at=data["updated_at"],
        )


@dataclass
class CreateSegmentRequest:
    """Request payload for creating a segment."""

    name: str
    conditions: Dict[str, Any]
    description: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {"name": self.name, "conditions": self.conditions}
        if self.description is not None:
            data["description"] = self.description
        return data


@dataclass
class UpdateSegmentRequest:
    """Request payload for updating a segment."""

    name: Optional[str] = None
    description: Optional[str] = None
    conditions: Optional[Dict[str, Any]] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {}
        if self.name is not None:
            data["name"] = self.name
        if self.description is not None:
            data["description"] = self.description
        if self.conditions is not None:
            data["conditions"] = self.conditions
        return data


@dataclass
class ListSegmentsParams:
    """Parameters for listing segments."""

    page: Optional[int] = None
    limit: Optional[int] = None


@dataclass
class SegmentContactsResponse:
    """Response from getting segment contacts."""

    contacts: List[Contact]
    total: int

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "SegmentContactsResponse":
        """Create from API response dictionary."""
        return cls(
            contacts=[Contact.from_dict(c) for c in data["contacts"]],
            total=data["total"],
        )


@dataclass
class SegmentCountResponse:
    """Response from getting segment contact count."""

    count: int

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "SegmentCountResponse":
        """Create from API response dictionary."""
        return cls(count=data["count"])


# ============================================================================
# API Key Types
# ============================================================================


@dataclass
class ApiKey:
    """API key record."""

    id: str
    organization_id: str
    name: str
    key: Optional[str]  # Only returned when creating
    permissions: Optional[List[str]]
    last_used_at: Optional[str]
    created_by: Optional[str]
    created_at: str
    revoked_at: Optional[str]

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ApiKey":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            organization_id=data["organization_id"],
            name=data["name"],
            key=data.get("key"),
            permissions=data.get("permissions"),
            last_used_at=data.get("last_used_at"),
            created_by=data.get("created_by"),
            created_at=data["created_at"],
            revoked_at=data.get("revoked_at"),
        )


@dataclass
class CreateApiKeyRequest:
    """Request payload for creating an API key."""

    name: str
    permissions: Optional[List[str]] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {"name": self.name}
        if self.permissions is not None:
            data["permissions"] = self.permissions
        return data


@dataclass
class ListApiKeysParams:
    """Parameters for listing API keys."""

    page: Optional[int] = None
    limit: Optional[int] = None


@dataclass
class ApiKeyListItem:
    """API key list item (without full key)."""

    id: str
    name: str
    permissions: Optional[List[str]]
    last_used_at: Optional[str]
    created_at: str

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ApiKeyListItem":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            name=data["name"],
            permissions=data.get("permissions"),
            last_used_at=data.get("last_used_at"),
            created_at=data["created_at"],
        )


# ============================================================================
# SMTP Types
# ============================================================================


@dataclass
class SmtpPorts:
    """SMTP port configuration."""

    tls: int
    ssl: int
    tls_alternative: int

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "SmtpPorts":
        """Create from API response dictionary."""
        return cls(
            tls=data["tls"],
            ssl=data["ssl"],
            tls_alternative=data["tls_alternative"],
        )


@dataclass
class SmtpCredentials:
    """SMTP credentials."""

    server: str
    ports: SmtpPorts
    username: str
    password: str
    encryption: str
    note: str

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "SmtpCredentials":
        """Create from API response dictionary."""
        return cls(
            server=data["server"],
            ports=SmtpPorts.from_dict(data["ports"]),
            username=data["username"],
            password=data["password"],
            encryption=data["encryption"],
            note=data["note"],
        )


# ============================================================================
# Settings Types
# ============================================================================


@dataclass
class Organization:
    """Organization record."""

    id: str
    name: str
    slug: str
    plan: Optional[str]
    billing_email: Optional[str]
    stripe_customer_id: Optional[str]
    stripe_subscription_id: Optional[str]
    usage_limit: Optional[int]
    created_at: str
    updated_at: str
    ses_configuration_set: Optional[str]
    ses_tenant_name: Optional[str]
    ses_tenant_id: Optional[str]

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Organization":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            name=data["name"],
            slug=data["slug"],
            plan=data.get("plan"),
            billing_email=data.get("billing_email"),
            stripe_customer_id=data.get("stripe_customer_id"),
            stripe_subscription_id=data.get("stripe_subscription_id"),
            usage_limit=data.get("usage_limit"),
            created_at=data["created_at"],
            updated_at=data["updated_at"],
            ses_configuration_set=data.get("ses_configuration_set"),
            ses_tenant_name=data.get("ses_tenant_name"),
            ses_tenant_id=data.get("ses_tenant_id"),
        )


@dataclass
class UpdateOrganizationRequest:
    """Request payload for updating organization."""

    name: Optional[str] = None
    billing_email: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {}
        if self.name is not None:
            data["name"] = self.name
        if self.billing_email is not None:
            data["billing_email"] = self.billing_email
        return data


@dataclass
class Profile:
    """User profile."""

    id: str
    email: str
    full_name: Optional[str]
    avatar_url: Optional[str]

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Profile":
        """Create from API response dictionary."""
        return cls(
            id=data["id"],
            email=data["email"],
            full_name=data.get("full_name"),
            avatar_url=data.get("avatar_url"),
        )


@dataclass
class TeamMember:
    """Team member record."""

    id: str
    organization_id: str
    user_id: str
    role: Optional[str]
    created_at: str
    profiles: Optional[Profile]

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "TeamMember":
        """Create from API response dictionary."""
        profiles = None
        if data.get("profiles"):
            profiles = Profile.from_dict(data["profiles"])
        return cls(
            id=data["id"],
            organization_id=data["organization_id"],
            user_id=data["user_id"],
            role=data.get("role"),
            created_at=data["created_at"],
            profiles=profiles,
        )


@dataclass
class UnsubscribeSettings:
    """Unsubscribe settings."""

    id: Optional[str] = None
    organization_id: Optional[str] = None
    custom_message: Optional[str] = None
    redirect_url: Optional[str] = None
    one_click_enabled: Optional[bool] = None
    logo_url: Optional[str] = None
    primary_color: Optional[str] = None
    company_name: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "UnsubscribeSettings":
        """Create from API response dictionary."""
        return cls(
            id=data.get("id"),
            organization_id=data.get("organization_id"),
            custom_message=data.get("custom_message"),
            redirect_url=data.get("redirect_url"),
            one_click_enabled=data.get("one_click_enabled"),
            logo_url=data.get("logo_url"),
            primary_color=data.get("primary_color"),
            company_name=data.get("company_name"),
            created_at=data.get("created_at"),
            updated_at=data.get("updated_at"),
        )


@dataclass
class UpdateUnsubscribeSettingsRequest:
    """Request payload for updating unsubscribe settings."""

    custom_message: Optional[str] = None
    redirect_url: Optional[str] = None
    one_click_enabled: Optional[bool] = None
    logo_url: Optional[str] = None
    primary_color: Optional[str] = None
    company_name: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for API request."""
        data: Dict[str, Any] = {}
        if self.custom_message is not None:
            data["custom_message"] = self.custom_message
        if self.redirect_url is not None:
            data["redirect_url"] = self.redirect_url
        if self.one_click_enabled is not None:
            data["one_click_enabled"] = self.one_click_enabled
        if self.logo_url is not None:
            data["logo_url"] = self.logo_url
        if self.primary_color is not None:
            data["primary_color"] = self.primary_color
        if self.company_name is not None:
            data["company_name"] = self.company_name
        return data


# ============================================================================
# Metrics Types
# ============================================================================


@dataclass
class UsageMetric:
    """Daily usage metric."""

    date: str
    emails_sent: int

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "UsageMetric":
        """Create from API response dictionary."""
        return cls(
            date=data["date"],
            emails_sent=data["emails_sent"],
        )


@dataclass
class EmailMetrics:
    """Aggregated email metrics."""

    total: int
    sent: int
    delivered: int
    bounced: int
    complained: int

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "EmailMetrics":
        """Create from API response dictionary."""
        return cls(
            total=data["total"],
            sent=data["sent"],
            delivered=data["delivered"],
            bounced=data["bounced"],
            complained=data["complained"],
        )



"""
Metrics resource for usage and email metrics.
"""

from typing import List, Optional

from emailr.http import AsyncHttpClient, HttpClient
from emailr.types import UsageMetric, EmailMetrics


class MetricsResource:
    """Synchronous metrics resource."""

    def __init__(self, http: HttpClient) -> None:
        self._http = http

    def get_usage(self, days: int = 30) -> List[UsageMetric]:
        """
        Get usage metrics for a specified number of days.

        Args:
            days: Number of days to get metrics for (default: 30)

        Returns:
            List of daily usage metrics
        """
        data = self._http.get("/v1/metrics/usage", params={"days": str(days)})
        if isinstance(data, list):
            return [UsageMetric.from_dict(m) for m in data]
        return [UsageMetric.from_dict(m) for m in data.get("data", data)]

    def get_email_metrics(self) -> EmailMetrics:
        """
        Get aggregated email delivery metrics.

        Returns:
            Email metrics
        """
        data = self._http.get("/v1/metrics/emails")
        return EmailMetrics.from_dict(data)


class AsyncMetricsResource:
    """Asynchronous metrics resource."""

    def __init__(self, http: AsyncHttpClient) -> None:
        self._http = http

    async def get_usage(self, days: int = 30) -> List[UsageMetric]:
        """Get usage metrics for a specified number of days."""
        data = await self._http.get("/v1/metrics/usage", params={"days": str(days)})
        if isinstance(data, list):
            return [UsageMetric.from_dict(m) for m in data]
        return [UsageMetric.from_dict(m) for m in data.get("data", data)]

    async def get_email_metrics(self) -> EmailMetrics:
        """Get aggregated email delivery metrics."""
        data = await self._http.get("/v1/metrics/emails")
        return EmailMetrics.from_dict(data)



"""
HTTP client implementations for the Emailr SDK.

Provides both synchronous (HttpClient) and asynchronous (AsyncHttpClient)
HTTP clients using httpx.
"""

from typing import Any, Dict, Optional, TypeVar, Union

import httpx

from emailr.errors import (
    AuthenticationError,
    EmailrError,
    NetworkError,
    NotFoundError,
    RateLimitError,
    ValidationError,
)

T = TypeVar("T")


class HttpClient:
    """
    Synchronous HTTP client for making API requests.

    Uses httpx for HTTP requests with automatic error handling
    and response parsing.
    """

    def __init__(
        self,
        api_key: str,
        base_url: str,
        timeout: float,
    ) -> None:
        self._api_key = api_key
        self._base_url = base_url.rstrip("/")
        self._timeout = timeout
        self._client: Optional[httpx.Client] = None

    def _get_client(self) -> httpx.Client:
        """Get or create the httpx client."""
        if self._client is None:
            self._client = httpx.Client(
                base_url=self._base_url,
                timeout=self._timeout,
                headers={
                    "Authorization": f"Bearer {self._api_key}",
                    "Content-Type": "application/json",
                },
            )
        return self._client

    def close(self) -> None:
        """Close the HTTP client."""
        if self._client is not None:
            self._client.close()
            self._client = None

    def get(
        self,
        path: str,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make a GET request."""
        return self._request("GET", path, params=params)

    def post(
        self,
        path: str,
        data: Optional[Any] = None,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make a POST request."""
        return self._request("POST", path, data=data, params=params)

    def put(
        self,
        path: str,
        data: Optional[Any] = None,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make a PUT request."""
        return self._request("PUT", path, data=data, params=params)

    def patch(
        self,
        path: str,
        data: Optional[Any] = None,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make a PATCH request."""
        return self._request("PATCH", path, data=data, params=params)

    def delete(
        self,
        path: str,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make a DELETE request."""
        return self._request("DELETE", path, params=params)

    def _request(
        self,
        method: str,
        path: str,
        data: Optional[Any] = None,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make an HTTP request."""
        client = self._get_client()

        # Filter out None values from params
        filtered_params: Optional[Dict[str, Any]] = None
        if params:
            filtered_params = {k: v for k, v in params.items() if v is not None}

        try:
            response = client.request(
                method=method,
                url=path,
                json=data if data is not None else None,
                params=filtered_params,
            )
        except httpx.TimeoutException as e:
            raise NetworkError(f"Request timeout after {self._timeout}s", cause=e)
        except httpx.RequestError as e:
            raise NetworkError(str(e), cause=e)

        request_id = response.headers.get("x-request-id")

        if not response.is_success:
            self._handle_error_response(response, request_id)

        # Handle empty responses (204 No Content)
        if response.status_code == 204:
            return {}

        return response.json()

    def _handle_error_response(
        self,
        response: httpx.Response,
        request_id: Optional[str],
    ) -> None:
        """Handle error responses and raise appropriate exceptions."""
        try:
            error_data = response.json()
        except Exception:
            error_data = {"error": response.text or "Unknown error"}

        message = error_data.get("error", "An error occurred")
        details = error_data.get("details")
        code = error_data.get("code")

        if response.status_code == 400:
            raise ValidationError(message, details=details, request_id=request_id)
        elif response.status_code == 401:
            raise AuthenticationError(message, request_id=request_id)
        elif response.status_code == 404:
            raise NotFoundError(message, request_id=request_id)
        elif response.status_code == 429:
            retry_after_header = response.headers.get("retry-after")
            retry_after = int(retry_after_header) if retry_after_header else None
            raise RateLimitError(message, retry_after=retry_after, request_id=request_id)
        else:
            raise EmailrError(
                message,
                status_code=response.status_code,
                code=code,
                request_id=request_id,
            )


class AsyncHttpClient:
    """
    Asynchronous HTTP client for making API requests.

    Uses httpx for async HTTP requests with automatic error handling
    and response parsing.
    """

    def __init__(
        self,
        api_key: str,
        base_url: str,
        timeout: float,
    ) -> None:
        self._api_key = api_key
        self._base_url = base_url.rstrip("/")
        self._timeout = timeout
        self._client: Optional[httpx.AsyncClient] = None

    async def _get_client(self) -> httpx.AsyncClient:
        """Get or create the async httpx client."""
        if self._client is None:
            self._client = httpx.AsyncClient(
                base_url=self._base_url,
                timeout=self._timeout,
                headers={
                    "Authorization": f"Bearer {self._api_key}",
                    "Content-Type": "application/json",
                },
            )
        return self._client

    async def close(self) -> None:
        """Close the HTTP client."""
        if self._client is not None:
            await self._client.aclose()
            self._client = None

    async def get(
        self,
        path: str,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make a GET request."""
        return await self._request("GET", path, params=params)

    async def post(
        self,
        path: str,
        data: Optional[Any] = None,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make a POST request."""
        return await self._request("POST", path, data=data, params=params)

    async def put(
        self,
        path: str,
        data: Optional[Any] = None,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make a PUT request."""
        return await self._request("PUT", path, data=data, params=params)

    async def patch(
        self,
        path: str,
        data: Optional[Any] = None,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make a PATCH request."""
        return await self._request("PATCH", path, data=data, params=params)

    async def delete(
        self,
        path: str,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make a DELETE request."""
        return await self._request("DELETE", path, params=params)

    async def _request(
        self,
        method: str,
        path: str,
        data: Optional[Any] = None,
        params: Optional[Dict[str, Union[str, int, bool, None]]] = None,
    ) -> Any:
        """Make an HTTP request."""
        client = await self._get_client()

        # Filter out None values from params
        filtered_params: Optional[Dict[str, Any]] = None
        if params:
            filtered_params = {k: v for k, v in params.items() if v is not None}

        try:
            response = await client.request(
                method=method,
                url=path,
                json=data if data is not None else None,
                params=filtered_params,
            )
        except httpx.TimeoutException as e:
            raise NetworkError(f"Request timeout after {self._timeout}s", cause=e)
        except httpx.RequestError as e:
            raise NetworkError(str(e), cause=e)

        request_id = response.headers.get("x-request-id")

        if not response.is_success:
            self._handle_error_response(response, request_id)

        # Handle empty responses (204 No Content)
        if response.status_code == 204:
            return {}

        return response.json()

    def _handle_error_response(
        self,
        response: httpx.Response,
        request_id: Optional[str],
    ) -> None:
        """Handle error responses and raise appropriate exceptions."""
        try:
            error_data = response.json()
        except Exception:
            error_data = {"error": response.text or "Unknown error"}

        message = error_data.get("error", "An error occurred")
        details = error_data.get("details")
        code = error_data.get("code")

        if response.status_code == 400:
            raise ValidationError(message, details=details, request_id=request_id)
        elif response.status_code == 401:
            raise AuthenticationError(message, request_id=request_id)
        elif response.status_code == 404:
            raise NotFoundError(message, request_id=request_id)
        elif response.status_code == 429:
            retry_after_header = response.headers.get("retry-after")
            retry_after = int(retry_after_header) if retry_after_header else None
            raise RateLimitError(message, retry_after=retry_after, request_id=request_id)
        else:
            raise EmailrError(
                message,
                status_code=response.status_code,
                code=code,
                request_id=request_id,
            )



"""
Tests for the synchronous Emailr client.
"""

import pytest
from unittest.mock import MagicMock, patch
import httpx

from emailr import Emailr
from emailr.errors import (
    EmailrError,
    AuthenticationError,
    NotFoundError,
    RateLimitError,
    ValidationError,
)


class TestEmailrClient:
    """Tests for the Emailr sync client."""

    def test_create_client_with_api_key(self, api_key):
        """Should create a client with API key."""
        client = Emailr(api_key=api_key)
        
        assert client is not None
        assert client._api_key == api_key

    def test_create_client_with_custom_base_url(self, api_key, base_url):
        """Should create a client with custom base URL."""
        client = Emailr(api_key=api_key, base_url=base_url)
        
        assert client._base_url == base_url

    def test_client_has_resources(self, emailr_client):
        """Should have all resource accessors."""
        assert hasattr(emailr_client, 'emails')
        assert hasattr(emailr_client, 'contacts')
        assert hasattr(emailr_client, 'templates')
        assert hasattr(emailr_client, 'domains')
        assert hasattr(emailr_client, 'webhooks')
        assert hasattr(emailr_client, 'broadcasts')
        assert hasattr(emailr_client, 'segments')


class TestEmailsResource:
    """Tests for the emails resource."""

    @patch('emailr.http.httpx.Client')
    def test_send_email(self, mock_client_class, api_key, sample_email):
        """Should send an email."""
        mock_client = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.is_success = True
        mock_response.json.return_value = sample_email
        mock_client.request.return_value = mock_response
        mock_client_class.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_class.return_value.__exit__ = MagicMock(return_value=False)

        client = Emailr(api_key=api_key)
        result = client.emails.send(
            from_address="sender@example.com",
            to=["recipient@example.com"],
            subject="Test Email",
            html="<p>Hello!</p>"
        )

        assert result is not None
        mock_client.request.assert_called_once()

    @patch('emailr.http.httpx.Client')
    def test_get_email(self, mock_client_class, api_key, sample_email):
        """Should get an email by ID."""
        mock_client = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.is_success = True
        mock_response.json.return_value = sample_email
        mock_client.request.return_value = mock_response
        mock_client_class.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_class.return_value.__exit__ = MagicMock(return_value=False)

        client = Emailr(api_key=api_key)
        result = client.emails.get("email_123")

        assert result is not None

    @patch('emailr.http.httpx.Client')
    def test_list_emails(self, mock_client_class, api_key, sample_email):
        """Should list emails."""
        mock_client = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.is_success = True
        mock_response.json.return_value = {"data": [sample_email], "pagination": {}}
        mock_client.request.return_value = mock_response
        mock_client_class.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_class.return_value.__exit__ = MagicMock(return_value=False)

        client = Emailr(api_key=api_key)
        result = client.emails.list()

        assert result is not None


class TestContactsResource:
    """Tests for the contacts resource."""

    @patch('emailr.http.httpx.Client')
    def test_create_contact(self, mock_client_class, api_key, sample_contact):
        """Should create a contact."""
        mock_client = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 201
        mock_response.is_success = True
        mock_response.json.return_value = sample_contact
        mock_client.request.return_value = mock_response
        mock_client_class.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_class.return_value.__exit__ = MagicMock(return_value=False)

        client = Emailr(api_key=api_key)
        result = client.contacts.create(
            email="user@example.com",
            first_name="John",
            last_name="Doe"
        )

        assert result is not None

    @patch('emailr.http.httpx.Client')
    def test_get_contact(self, mock_client_class, api_key, sample_contact):
        """Should get a contact by ID."""
        mock_client = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_response.is_success = True
        mock_response.json.return_value = sample_contact
        mock_client.request.return_value = mock_response
        mock_client_class.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_class.return_value.__exit__ = MagicMock(return_value=False)

        client = Emailr(api_key=api_key)
        result = client.contacts.get("contact_123")

        assert result is not None

    @patch('emailr.http.httpx.Client')
    def test_delete_contact(self, mock_client_class, api_key):
        """Should delete a contact."""
        mock_client = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 204
        mock_response.is_success = True
        mock_client.request.return_value = mock_response
        mock_client_class.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_class.return_value.__exit__ = MagicMock(return_value=False)

        client = Emailr(api_key=api_key)
        client.contacts.delete("contact_123")

        mock_client.request.assert_called_once()


class TestErrorHandling:
    """Tests for error handling in the client."""

    @patch('emailr.http.httpx.Client')
    def test_authentication_error(self, mock_client_class, api_key):
        """Should raise AuthenticationError for 401 response."""
        mock_client = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 401
        mock_response.is_success = False
        mock_response.json.return_value = {"error": {"message": "Invalid API key"}}
        mock_client.request.return_value = mock_response
        mock_client_class.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_class.return_value.__exit__ = MagicMock(return_value=False)

        client = Emailr(api_key=api_key)
        
        with pytest.raises(AuthenticationError):
            client.emails.list()

    @patch('emailr.http.httpx.Client')
    def test_not_found_error(self, mock_client_class, api_key):
        """Should raise NotFoundError for 404 response."""
        mock_client = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 404
        mock_response.is_success = False
        mock_response.json.return_value = {"error": {"message": "Not found"}}
        mock_client.request.return_value = mock_response
        mock_client_class.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_class.return_value.__exit__ = MagicMock(return_value=False)

        client = Emailr(api_key=api_key)
        
        with pytest.raises(NotFoundError):
            client.emails.get("nonexistent")

    @patch('emailr.http.httpx.Client')
    def test_rate_limit_error(self, mock_client_class, api_key):
        """Should raise RateLimitError for 429 response."""
        mock_client = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 429
        mock_response.is_success = False
        mock_response.headers = {"Retry-After": "60"}
        mock_response.json.return_value = {"error": {"message": "Rate limit exceeded"}}
        mock_client.request.return_value = mock_response
        mock_client_class.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_class.return_value.__exit__ = MagicMock(return_value=False)

        client = Emailr(api_key=api_key)
        
        with pytest.raises(RateLimitError) as exc_info:
            client.emails.list()
        
        assert exc_info.value.retry_after == 60

    @patch('emailr.http.httpx.Client')
    def test_validation_error(self, mock_client_class, api_key):
        """Should raise ValidationError for 400 response."""
        mock_client = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 400
        mock_response.is_success = False
        mock_response.json.return_value = {
            "error": {
                "message": "Validation failed",
                "details": {"email": ["Invalid format"]}
            }
        }
        mock_client.request.return_value = mock_response
        mock_client_class.return_value.__enter__ = MagicMock(return_value=mock_client)
        mock_client_class.return_value.__exit__ = MagicMock(return_value=False)

        client = Emailr(api_key=api_key)
        
        with pytest.raises(ValidationError) as exc_info:
            client.contacts.create(email="invalid")
        
        assert exc_info.value.details == {"email": ["Invalid format"]}



"""
Email resource for sending and managing emails.
"""

from typing import Any, Dict, List, Optional, Union

from emailr.http import AsyncHttpClient, HttpClient
from emailr.types import (
    CreateForwardingRuleRequest,
    Email,
    ForwardEmailRequest,
    ForwardingRule,
    ListEmailsParams,
    PaginatedResponse,
    Pagination,
    SendEmailRequest,
    SendEmailResponse,
    SuccessResponse,
)


class EmailsResource:
    """Synchronous email resource for sending and managing emails."""

    def __init__(self, http: HttpClient) -> None:
        self._http = http

    def send(
        self,
        to: Union[str, List[str]],
        *,
        from_: Optional[str] = None,
        cc: Optional[Union[str, List[str]]] = None,
        bcc: Optional[Union[str, List[str]]] = None,
        subject: Optional[str] = None,
        html: Optional[str] = None,
        text: Optional[str] = None,
        template_id: Optional[str] = None,
        template_data: Optional[Dict[str, Any]] = None,
        tags: Optional[Dict[str, str]] = None,
        scheduled_at: Optional[str] = None,
    ) -> SendEmailResponse:
        """
        Send an email to one or multiple recipients.

        Args:
            to: Recipient email address(es)
            from_: Sender email address
            cc: CC recipient(s)
            bcc: BCC recipient(s)
            subject: Email subject
            html: HTML content
            text: Plain text content
            template_id: Template ID to use
            template_data: Data for template variables
            tags: Custom tags for tracking
            scheduled_at: ISO 8601 datetime to schedule sending

        Returns:
            SendEmailResponse with message_id and status
        """
        request = SendEmailRequest(
            to=to,
            from_=from_,
            cc=cc,
            bcc=bcc,
            subject=subject,
            html=html,
            text=text,
            template_id=template_id,
            template_data=template_data,
            tags=tags,
            scheduled_at=scheduled_at,
        )
        data = self._http.post("/v1/emails/send", request.to_dict())
        return SendEmailResponse.from_dict(data)


    def get(self, id: str) -> Email:
        """
        Get email by ID.

        Args:
            id: Email ID

        Returns:
            Email record
        """
        data = self._http.get(f"/v1/emails/{id}")
        return Email.from_dict(data)

    def list(
        self,
        *,
        page: Optional[int] = None,
        limit: Optional[int] = None,
    ) -> PaginatedResponse[Email]:
        """
        List emails with pagination.

        Args:
            page: Page number
            limit: Number of items per page

        Returns:
            Paginated list of emails
        """
        params: Dict[str, Any] = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit

        data = self._http.get("/v1/emails", params=params)
        return PaginatedResponse(
            data=[Email.from_dict(e) for e in data["data"]],
            pagination=Pagination(**data["pagination"]),
        )

    def forward(
        self,
        email_id: str,
        to: Union[str, List[str]],
        *,
        message: Optional[str] = None,
    ) -> SendEmailResponse:
        """
        Forward an email to other recipients.

        Args:
            email_id: ID of the email to forward
            to: Recipient email address(es)
            message: Optional message to include

        Returns:
            SendEmailResponse with message_id and status
        """
        request = ForwardEmailRequest(email_id=email_id, to=to, message=message)
        data = self._http.post("/v1/emails/forward", request.to_dict())
        return SendEmailResponse.from_dict(data)

    def create_forwarding_rule(
        self,
        name: str,
        from_pattern: str,
        to_addresses: List[str],
        *,
        active: Optional[bool] = None,
    ) -> Dict[str, Any]:
        """
        Create an email forwarding rule.

        Args:
            name: Rule name
            from_pattern: Pattern to match sender
            to_addresses: Addresses to forward to
            active: Whether the rule is active

        Returns:
            Dict with id and success status
        """
        request = CreateForwardingRuleRequest(
            name=name,
            from_pattern=from_pattern,
            to_addresses=to_addresses,
            active=active,
        )
        return self._http.post("/v1/emails/forwarding-rules", request.to_dict())

    def list_forwarding_rules(self) -> List[ForwardingRule]:
        """
        List all forwarding rules.

        Returns:
            List of forwarding rules
        """
        data = self._http.get("/v1/emails/forwarding-rules")
        return [ForwardingRule.from_dict(r) for r in data["data"]]

    def delete_forwarding_rule(self, id: str) -> SuccessResponse:
        """
        Delete a forwarding rule.

        Args:
            id: Forwarding rule ID

        Returns:
            Success response
        """
        data = self._http.delete(f"/v1/emails/forwarding-rules/{id}")
        return SuccessResponse(success=data.get("success", True))


class AsyncEmailsResource:
    """Asynchronous email resource for sending and managing emails."""

    def __init__(self, http: AsyncHttpClient) -> None:
        self._http = http

    async def send(
        self,
        to: Union[str, List[str]],
        *,
        from_: Optional[str] = None,
        cc: Optional[Union[str, List[str]]] = None,
        bcc: Optional[Union[str, List[str]]] = None,
        subject: Optional[str] = None,
        html: Optional[str] = None,
        text: Optional[str] = None,
        template_id: Optional[str] = None,
        template_data: Optional[Dict[str, Any]] = None,
        tags: Optional[Dict[str, str]] = None,
        scheduled_at: Optional[str] = None,
    ) -> SendEmailResponse:
        """
        Send an email to one or multiple recipients.

        Args:
            to: Recipient email address(es)
            from_: Sender email address
            cc: CC recipient(s)
            bcc: BCC recipient(s)
            subject: Email subject
            html: HTML content
            text: Plain text content
            template_id: Template ID to use
            template_data: Data for template variables
            tags: Custom tags for tracking
            scheduled_at: ISO 8601 datetime to schedule sending

        Returns:
            SendEmailResponse with message_id and status
        """
        request = SendEmailRequest(
            to=to,
            from_=from_,
            cc=cc,
            bcc=bcc,
            subject=subject,
            html=html,
            text=text,
            template_id=template_id,
            template_data=template_data,
            tags=tags,
            scheduled_at=scheduled_at,
        )
        data = await self._http.post("/v1/emails/send", request.to_dict())
        return SendEmailResponse.from_dict(data)

    async def get(self, id: str) -> Email:
        """Get email by ID."""
        data = await self._http.get(f"/v1/emails/{id}")
        return Email.from_dict(data)

    async def list(
        self,
        *,
        page: Optional[int] = None,
        limit: Optional[int] = None,
    ) -> PaginatedResponse[Email]:
        """List emails with pagination."""
        params: Dict[str, Any] = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit

        data = await self._http.get("/v1/emails", params=params)
        return PaginatedResponse(
            data=[Email.from_dict(e) for e in data["data"]],
            pagination=Pagination(**data["pagination"]),
        )

    async def forward(
        self,
        email_id: str,
        to: Union[str, List[str]],
        *,
        message: Optional[str] = None,
    ) -> SendEmailResponse:
        """Forward an email to other recipients."""
        request = ForwardEmailRequest(email_id=email_id, to=to, message=message)
        data = await self._http.post("/v1/emails/forward", request.to_dict())
        return SendEmailResponse.from_dict(data)

    async def create_forwarding_rule(
        self,
        name: str,
        from_pattern: str,
        to_addresses: List[str],
        *,
        active: Optional[bool] = None,
    ) -> Dict[str, Any]:
        """Create an email forwarding rule."""
        request = CreateForwardingRuleRequest(
            name=name,
            from_pattern=from_pattern,
            to_addresses=to_addresses,
            active=active,
        )
        return await self._http.post("/v1/emails/forwarding-rules", request.to_dict())

    async def list_forwarding_rules(self) -> List[ForwardingRule]:
        """List all forwarding rules."""
        data = await self._http.get("/v1/emails/forwarding-rules")
        return [ForwardingRule.from_dict(r) for r in data["data"]]

    async def delete_forwarding_rule(self, id: str) -> SuccessResponse:
        """Delete a forwarding rule."""
        data = await self._http.delete(f"/v1/emails/forwarding-rules/{id}")
        return SuccessResponse(success=data.get("success", True))



"""
Segment resource for managing contact segments.
"""

from typing import Any, Dict, List, Optional

from emailr.http import AsyncHttpClient, HttpClient
from emailr.types import (
    CreateSegmentRequest,
    Segment,
    SegmentContactsResponse,
    SegmentCountResponse,
    SuccessResponse,
    UpdateSegmentRequest,
)


class SegmentsResource:
    """Synchronous segment resource for managing contact segments."""

    def __init__(self, http: HttpClient) -> None:
        self._http = http

    def create(
        self,
        name: str,
        conditions: Dict[str, Any],
        *,
        description: Optional[str] = None,
    ) -> Segment:
        """
        Create a new segment.

        Args:
            name: Segment name
            conditions: Segment conditions/filters
            description: Segment description

        Returns:
            Created segment
        """
        request = CreateSegmentRequest(
            name=name,
            conditions=conditions,
            description=description,
        )
        data = self._http.post("/v1/segments", request.to_dict())
        return Segment.from_dict(data)

    def get(self, id: str) -> Segment:
        """
        Get segment by ID.

        Args:
            id: Segment ID

        Returns:
            Segment record
        """
        data = self._http.get(f"/v1/segments/{id}")
        return Segment.from_dict(data)

    def list(
        self,
        *,
        page: Optional[int] = None,
        limit: Optional[int] = None,
    ) -> List[Segment]:
        """
        List segments.

        Args:
            page: Page number
            limit: Number of items per page

        Returns:
            List of segments
        """
        params: Dict[str, Any] = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit

        data = self._http.get("/v1/segments", params=params)
        # API returns array directly
        if isinstance(data, list):
            return [Segment.from_dict(s) for s in data]
        return [Segment.from_dict(s) for s in data.get("data", data)]


    def update(
        self,
        id: str,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        conditions: Optional[Dict[str, Any]] = None,
    ) -> Segment:
        """
        Update a segment.

        Args:
            id: Segment ID
            name: New segment name
            description: New segment description
            conditions: New segment conditions

        Returns:
            Updated segment
        """
        request = UpdateSegmentRequest(
            name=name,
            description=description,
            conditions=conditions,
        )
        data = self._http.put(f"/v1/segments/{id}", request.to_dict())
        return Segment.from_dict(data)

    def delete(self, id: str) -> SuccessResponse:
        """
        Delete a segment.

        Args:
            id: Segment ID

        Returns:
            Success response
        """
        data = self._http.delete(f"/v1/segments/{id}")
        return SuccessResponse(success=data.get("success", True))

    def get_contacts(
        self,
        id: str,
        *,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
    ) -> SegmentContactsResponse:
        """
        Get contacts in a segment.

        Args:
            id: Segment ID
            limit: Number of contacts to return
            offset: Number of contacts to skip

        Returns:
            Segment contacts response
        """
        params: Dict[str, Any] = {}
        if limit is not None:
            params["limit"] = limit
        if offset is not None:
            params["offset"] = offset

        data = self._http.get(f"/v1/segments/{id}/contacts", params=params)
        return SegmentContactsResponse.from_dict(data)

    def get_count(self, id: str) -> SegmentCountResponse:
        """
        Get segment contact count.

        Args:
            id: Segment ID

        Returns:
            Segment count response
        """
        data = self._http.get(f"/v1/segments/{id}/count")
        return SegmentCountResponse.from_dict(data)


class AsyncSegmentsResource:
    """Asynchronous segment resource for managing contact segments."""

    def __init__(self, http: AsyncHttpClient) -> None:
        self._http = http

    async def create(
        self,
        name: str,
        conditions: Dict[str, Any],
        *,
        description: Optional[str] = None,
    ) -> Segment:
        """Create a new segment."""
        request = CreateSegmentRequest(
            name=name,
            conditions=conditions,
            description=description,
        )
        data = await self._http.post("/v1/segments", request.to_dict())
        return Segment.from_dict(data)

    async def get(self, id: str) -> Segment:
        """Get segment by ID."""
        data = await self._http.get(f"/v1/segments/{id}")
        return Segment.from_dict(data)

    async def list(
        self,
        *,
        page: Optional[int] = None,
        limit: Optional[int] = None,
    ) -> List[Segment]:
        """List segments."""
        params: Dict[str, Any] = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit

        data = await self._http.get("/v1/segments", params=params)
        # API returns array directly
        if isinstance(data, list):
            return [Segment.from_dict(s) for s in data]
        return [Segment.from_dict(s) for s in data.get("data", data)]

    async def update(
        self,
        id: str,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        conditions: Optional[Dict[str, Any]] = None,
    ) -> Segment:
        """Update a segment."""
        request = UpdateSegmentRequest(
            name=name,
            description=description,
            conditions=conditions,
        )
        data = await self._http.put(f"/v1/segments/{id}", request.to_dict())
        return Segment.from_dict(data)

    async def delete(self, id: str) -> SuccessResponse:
        """Delete a segment."""
        data = await self._http.delete(f"/v1/segments/{id}")
        return SuccessResponse(success=data.get("success", True))

    async def get_contacts(
        self,
        id: str,
        *,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
    ) -> SegmentContactsResponse:
        """Get contacts in a segment."""
        params: Dict[str, Any] = {}
        if limit is not None:
            params["limit"] = limit
        if offset is not None:
            params["offset"] = offset

        data = await self._http.get(f"/v1/segments/{id}/contacts", params=params)
        return SegmentContactsResponse.from_dict(data)

    async def get_count(self, id: str) -> SegmentCountResponse:
        """Get segment contact count."""
        data = await self._http.get(f"/v1/segments/{id}/count")
        return SegmentCountResponse.from_dict(data)



"""
Resource modules for the Emailr SDK.

Each resource provides both synchronous and asynchronous methods
for interacting with the corresponding API endpoints.
"""

from emailr.resources.broadcasts import AsyncBroadcastsResource, BroadcastsResource
from emailr.resources.contacts import AsyncContactsResource, ContactsResource
from emailr.resources.domains import AsyncDomainsResource, DomainsResource
from emailr.resources.emails import AsyncEmailsResource, EmailsResource
from emailr.resources.segments import AsyncSegmentsResource, SegmentsResource
from emailr.resources.templates import AsyncTemplatesResource, TemplatesResource
from emailr.resources.webhooks import AsyncWebhooksResource, WebhooksResource
from emailr.resources.api_keys import AsyncApiKeysResource, ApiKeysResource
from emailr.resources.smtp import AsyncSmtpResource, SmtpResource
from emailr.resources.settings import AsyncSettingsResource, SettingsResource
from emailr.resources.metrics import AsyncMetricsResource, MetricsResource

__all__ = [
    # Sync resources
    "EmailsResource",
    "ContactsResource",
    "TemplatesResource",
    "DomainsResource",
    "WebhooksResource",
    "BroadcastsResource",
    "SegmentsResource",
    "ApiKeysResource",
    "SmtpResource",
    "SettingsResource",
    "MetricsResource",
    # Async resources
    "AsyncEmailsResource",
    "AsyncContactsResource",
    "AsyncTemplatesResource",
    "AsyncDomainsResource",
    "AsyncWebhooksResource",
    "AsyncBroadcastsResource",
    "AsyncSegmentsResource",
    "AsyncApiKeysResource",
    "AsyncSmtpResource",
    "AsyncSettingsResource",
    "AsyncMetricsResource",
]



"""
Broadcast resource for managing broadcast campaigns.
"""

from typing import Any, Dict, List, Optional

from emailr.http import AsyncHttpClient, HttpClient
from emailr.types import (
    Broadcast,
    BroadcastStats,
    CreateBroadcastRequest,
    SendBroadcastResponse,
    SuccessResponse,
    UpdateBroadcastRequest,
)


class BroadcastsResource:
    """Synchronous broadcast resource for managing broadcast campaigns."""

    def __init__(self, http: HttpClient) -> None:
        self._http = http

    def create(
        self,
        name: str,
        subject: str,
        from_email: str,
        *,
        template_id: Optional[str] = None,
        segment_id: Optional[str] = None,
        html_content: Optional[str] = None,
        text_content: Optional[str] = None,
        scheduled_at: Optional[str] = None,
    ) -> Broadcast:
        """
        Create a new broadcast.

        Args:
            name: Broadcast name
            subject: Email subject
            from_email: Sender email address
            template_id: Template ID to use
            segment_id: Segment ID to target
            html_content: HTML content (if not using template)
            text_content: Plain text content
            scheduled_at: ISO 8601 datetime to schedule

        Returns:
            Created broadcast
        """
        request = CreateBroadcastRequest(
            name=name,
            subject=subject,
            from_email=from_email,
            template_id=template_id,
            segment_id=segment_id,
            html_content=html_content,
            text_content=text_content,
            scheduled_at=scheduled_at,
        )
        data = self._http.post("/v1/broadcasts", request.to_dict())
        return Broadcast.from_dict(data)


    def get(self, id: str) -> Broadcast:
        """
        Get broadcast by ID.

        Args:
            id: Broadcast ID

        Returns:
            Broadcast record
        """
        data = self._http.get(f"/v1/broadcasts/{id}")
        return Broadcast.from_dict(data)

    def list(
        self,
        *,
        page: Optional[int] = None,
        limit: Optional[int] = None,
        status: Optional[str] = None,
    ) -> List[Broadcast]:
        """
        List broadcasts.

        Args:
            page: Page number
            limit: Number of items per page
            status: Filter by status

        Returns:
            List of broadcasts
        """
        params: Dict[str, Any] = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit
        if status is not None:
            params["status"] = status

        data = self._http.get("/v1/broadcasts", params=params)
        # API returns array directly
        if isinstance(data, list):
            return [Broadcast.from_dict(b) for b in data]
        return [Broadcast.from_dict(b) for b in data.get("data", data)]

    def update(
        self,
        id: str,
        *,
        name: Optional[str] = None,
        subject: Optional[str] = None,
        from_email: Optional[str] = None,
        template_id: Optional[str] = None,
        segment_id: Optional[str] = None,
        html_content: Optional[str] = None,
        text_content: Optional[str] = None,
        scheduled_at: Optional[str] = None,
    ) -> Broadcast:
        """
        Update a broadcast.

        Args:
            id: Broadcast ID
            name: New broadcast name
            subject: New email subject
            from_email: New sender email
            template_id: New template ID
            segment_id: New segment ID
            html_content: New HTML content
            text_content: New plain text content
            scheduled_at: New scheduled datetime

        Returns:
            Updated broadcast
        """
        request = UpdateBroadcastRequest(
            name=name,
            subject=subject,
            from_email=from_email,
            template_id=template_id,
            segment_id=segment_id,
            html_content=html_content,
            text_content=text_content,
            scheduled_at=scheduled_at,
        )
        data = self._http.put(f"/v1/broadcasts/{id}", request.to_dict())
        return Broadcast.from_dict(data)

    def delete(self, id: str) -> SuccessResponse:
        """
        Delete a broadcast.

        Args:
            id: Broadcast ID

        Returns:
            Success response
        """
        data = self._http.delete(f"/v1/broadcasts/{id}")
        return SuccessResponse(success=data.get("success", True))

    def send(self, id: str) -> SendBroadcastResponse:
        """
        Send a broadcast immediately.

        Args:
            id: Broadcast ID

        Returns:
            Send response with counts
        """
        data = self._http.post(f"/v1/broadcasts/{id}/send")
        return SendBroadcastResponse.from_dict(data)

    def schedule(self, id: str, scheduled_at: str) -> Broadcast:
        """
        Schedule a broadcast.

        Args:
            id: Broadcast ID
            scheduled_at: ISO 8601 datetime to schedule

        Returns:
            Updated broadcast
        """
        data = self._http.post(f"/v1/broadcasts/{id}/schedule", {"scheduled_at": scheduled_at})
        return Broadcast.from_dict(data)

    def cancel(self, id: str) -> SuccessResponse:
        """
        Cancel a scheduled broadcast.

        Args:
            id: Broadcast ID

        Returns:
            Success response
        """
        data = self._http.post(f"/v1/broadcasts/{id}/cancel")
        return SuccessResponse(success=data.get("success", True))

    def get_stats(self, id: str) -> BroadcastStats:
        """
        Get broadcast statistics.

        Args:
            id: Broadcast ID

        Returns:
            Broadcast statistics
        """
        data = self._http.get(f"/v1/broadcasts/{id}/stats")
        return BroadcastStats.from_dict(data)


class AsyncBroadcastsResource:
    """Asynchronous broadcast resource for managing broadcast campaigns."""

    def __init__(self, http: AsyncHttpClient) -> None:
        self._http = http

    async def create(
        self,
        name: str,
        subject: str,
        from_email: str,
        *,
        template_id: Optional[str] = None,
        segment_id: Optional[str] = None,
        html_content: Optional[str] = None,
        text_content: Optional[str] = None,
        scheduled_at: Optional[str] = None,
    ) -> Broadcast:
        """Create a new broadcast."""
        request = CreateBroadcastRequest(
            name=name,
            subject=subject,
            from_email=from_email,
            template_id=template_id,
            segment_id=segment_id,
            html_content=html_content,
            text_content=text_content,
            scheduled_at=scheduled_at,
        )
        data = await self._http.post("/v1/broadcasts", request.to_dict())
        return Broadcast.from_dict(data)

    async def get(self, id: str) -> Broadcast:
        """Get broadcast by ID."""
        data = await self._http.get(f"/v1/broadcasts/{id}")
        return Broadcast.from_dict(data)

    async def list(
        self,
        *,
        page: Optional[int] = None,
        limit: Optional[int] = None,
        status: Optional[str] = None,
    ) -> List[Broadcast]:
        """List broadcasts."""
        params: Dict[str, Any] = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit
        if status is not None:
            params["status"] = status

        data = await self._http.get("/v1/broadcasts", params=params)
        # API returns array directly
        if isinstance(data, list):
            return [Broadcast.from_dict(b) for b in data]
        return [Broadcast.from_dict(b) for b in data.get("data", data)]

    async def update(
        self,
        id: str,
        *,
        name: Optional[str] = None,
        subject: Optional[str] = None,
        from_email: Optional[str] = None,
        template_id: Optional[str] = None,
        segment_id: Optional[str] = None,
        html_content: Optional[str] = None,
        text_content: Optional[str] = None,
        scheduled_at: Optional[str] = None,
    ) -> Broadcast:
        """Update a broadcast."""
        request = UpdateBroadcastRequest(
            name=name,
            subject=subject,
            from_email=from_email,
            template_id=template_id,
            segment_id=segment_id,
            html_content=html_content,
            text_content=text_content,
            scheduled_at=scheduled_at,
        )
        data = await self._http.put(f"/v1/broadcasts/{id}", request.to_dict())
        return Broadcast.from_dict(data)

    async def delete(self, id: str) -> SuccessResponse:
        """Delete a broadcast."""
        data = await self._http.delete(f"/v1/broadcasts/{id}")
        return SuccessResponse(success=data.get("success", True))

    async def send(self, id: str) -> SendBroadcastResponse:
        """Send a broadcast immediately."""
        data = await self._http.post(f"/v1/broadcasts/{id}/send")
        return SendBroadcastResponse.from_dict(data)

    async def schedule(self, id: str, scheduled_at: str) -> Broadcast:
        """Schedule a broadcast."""
        data = await self._http.post(
            f"/v1/broadcasts/{id}/schedule", {"scheduled_at": scheduled_at}
        )
        return Broadcast.from_dict(data)

    async def cancel(self, id: str) -> SuccessResponse:
        """Cancel a scheduled broadcast."""
        data = await self._http.post(f"/v1/broadcasts/{id}/cancel")
        return SuccessResponse(success=data.get("success", True))

    async def get_stats(self, id: str) -> BroadcastStats:
        """Get broadcast statistics."""
        data = await self._http.get(f"/v1/broadcasts/{id}/stats")
        return BroadcastStats.from_dict(data)



"""
Template resource for managing email templates.
"""

from typing import Any, Dict, List, Optional

from emailr.http import AsyncHttpClient, HttpClient
from emailr.types import (
    CreateTemplateRequest,
    PaginatedResponse,
    Pagination,
    SuccessResponse,
    Template,
    UpdateTemplateRequest,
)


class TemplatesResource:
    """Synchronous template resource for managing email templates."""

    def __init__(self, http: HttpClient) -> None:
        self._http = http

    def create(
        self,
        name: str,
        subject: str,
        *,
        html_content: Optional[str] = None,
        text_content: Optional[str] = None,
        variables: Optional[List[str]] = None,
        from_email: Optional[str] = None,
        reply_to: Optional[str] = None,
        preview_text: Optional[str] = None,
    ) -> Template:
        """
        Create a new template.

        Args:
            name: Template name
            subject: Email subject
            html_content: HTML content
            text_content: Plain text content
            variables: List of variable names
            from_email: Default sender email
            reply_to: Default reply-to email
            preview_text: Email preview text

        Returns:
            Created template
        """
        request = CreateTemplateRequest(
            name=name,
            subject=subject,
            html_content=html_content,
            text_content=text_content,
            variables=variables,
            from_email=from_email,
            reply_to=reply_to,
            preview_text=preview_text,
        )
        data = self._http.post("/v1/templates", request.to_dict())
        return Template.from_dict(data)


    def get(self, id: str) -> Template:
        """
        Get template by ID.

        Args:
            id: Template ID

        Returns:
            Template record
        """
        data = self._http.get(f"/v1/templates/{id}")
        return Template.from_dict(data)

    def list(
        self,
        *,
        page: Optional[int] = None,
        limit: Optional[int] = None,
    ) -> List[Template]:
        """
        List templates with pagination.

        Args:
            page: Page number
            limit: Number of items per page

        Returns:
            List of templates
        """
        params: Dict[str, Any] = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit

        data = self._http.get("/v1/templates", params=params)
        # API returns array directly
        if isinstance(data, list):
            return [Template.from_dict(t) for t in data]
        # Or wrapped in data/pagination
        return [Template.from_dict(t) for t in data.get("data", data)]

    def update(
        self,
        id: str,
        *,
        name: Optional[str] = None,
        subject: Optional[str] = None,
        html_content: Optional[str] = None,
        text_content: Optional[str] = None,
        variables: Optional[List[str]] = None,
        from_email: Optional[str] = None,
        reply_to: Optional[str] = None,
        preview_text: Optional[str] = None,
    ) -> Template:
        """
        Update a template.

        Args:
            id: Template ID
            name: New template name
            subject: New email subject
            html_content: New HTML content
            text_content: New plain text content
            variables: New list of variable names
            from_email: New default sender email
            reply_to: New default reply-to email
            preview_text: New email preview text

        Returns:
            Updated template
        """
        request = UpdateTemplateRequest(
            name=name,
            subject=subject,
            html_content=html_content,
            text_content=text_content,
            variables=variables,
            from_email=from_email,
            reply_to=reply_to,
            preview_text=preview_text,
        )
        data = self._http.put(f"/v1/templates/{id}", request.to_dict())
        return Template.from_dict(data)

    def delete(self, id: str) -> SuccessResponse:
        """
        Delete a template.

        Args:
            id: Template ID

        Returns:
            Success response
        """
        data = self._http.delete(f"/v1/templates/{id}")
        return SuccessResponse(success=data.get("success", True))

    def duplicate(self, id: str) -> Template:
        """
        Duplicate a template.

        Args:
            id: Template ID to duplicate

        Returns:
            New duplicated template
        """
        data = self._http.post(f"/v1/templates/{id}/duplicate")
        return Template.from_dict(data)


class AsyncTemplatesResource:
    """Asynchronous template resource for managing email templates."""

    def __init__(self, http: AsyncHttpClient) -> None:
        self._http = http

    async def create(
        self,
        name: str,
        subject: str,
        *,
        html_content: Optional[str] = None,
        text_content: Optional[str] = None,
        variables: Optional[List[str]] = None,
        from_email: Optional[str] = None,
        reply_to: Optional[str] = None,
        preview_text: Optional[str] = None,
    ) -> Template:
        """Create a new template."""
        request = CreateTemplateRequest(
            name=name,
            subject=subject,
            html_content=html_content,
            text_content=text_content,
            variables=variables,
            from_email=from_email,
            reply_to=reply_to,
            preview_text=preview_text,
        )
        data = await self._http.post("/v1/templates", request.to_dict())
        return Template.from_dict(data)

    async def get(self, id: str) -> Template:
        """Get template by ID."""
        data = await self._http.get(f"/v1/templates/{id}")
        return Template.from_dict(data)

    async def list(
        self,
        *,
        page: Optional[int] = None,
        limit: Optional[int] = None,
    ) -> List[Template]:
        """List templates with pagination."""
        params: Dict[str, Any] = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit

        data = await self._http.get("/v1/templates", params=params)
        # API returns array directly
        if isinstance(data, list):
            return [Template.from_dict(t) for t in data]
        # Or wrapped in data/pagination
        return [Template.from_dict(t) for t in data.get("data", data)]

    async def update(
        self,
        id: str,
        *,
        name: Optional[str] = None,
        subject: Optional[str] = None,
        html_content: Optional[str] = None,
        text_content: Optional[str] = None,
        variables: Optional[List[str]] = None,
        from_email: Optional[str] = None,
        reply_to: Optional[str] = None,
        preview_text: Optional[str] = None,
    ) -> Template:
        """Update a template."""
        request = UpdateTemplateRequest(
            name=name,
            subject=subject,
            html_content=html_content,
            text_content=text_content,
            variables=variables,
            from_email=from_email,
            reply_to=reply_to,
            preview_text=preview_text,
        )
        data = await self._http.put(f"/v1/templates/{id}", request.to_dict())
        return Template.from_dict(data)

    async def delete(self, id: str) -> SuccessResponse:
        """Delete a template."""
        data = await self._http.delete(f"/v1/templates/{id}")
        return SuccessResponse(success=data.get("success", True))

    async def duplicate(self, id: str) -> Template:
        """Duplicate a template."""
        data = await self._http.post(f"/v1/templates/{id}/duplicate")
        return Template.from_dict(data)



"""
Emailr SDK for Python

Official Python SDK for the Emailr email API service.
Provides both synchronous and asynchronous clients for sending emails,
managing contacts, templates, domains, webhooks, broadcasts, and segments.

Example usage:
    >>> from emailr import Emailr
    >>> client = Emailr(api_key="your-api-key")
    >>> result = client.emails.send(
    ...     to="recipient@example.com",
    ...     from_="sender@yourdomain.com",
    ...     subject="Hello!",
    ...     html="<h1>Hello World</h1>"
    ... )

For async usage:
    >>> from emailr import AsyncEmailr
    >>> async with AsyncEmailr(api_key="your-api-key") as client:
    ...     result = await client.emails.send(
    ...         to="recipient@example.com",
    ...         from_="sender@yourdomain.com",
    ...         subject="Hello!",
    ...         html="<h1>Hello World</h1>"
    ...     )
"""

from emailr.client import AsyncEmailr, Emailr
from emailr.errors import (
    AuthenticationError,
    EmailrError,
    NetworkError,
    NotFoundError,
    RateLimitError,
    ValidationError,
)
from emailr.types import (
    AddDomainRequest,
    ApiKey,
    Attachment,
    Broadcast,
    BroadcastStats,
    BulkCreateContactsRequest,
    BulkCreateContactsResponse,
    Contact,
    ContactListResponse,
    CreateApiKeyRequest,
    CreateBroadcastRequest,
    CreateContactRequest,
    CreateForwardingRuleRequest,
    CreateSegmentRequest,
    CreateTemplateRequest,
    CreateWebhookRequest,
    DnsRecord,
    DnsVerificationStatus,
    Domain,
    DomainDnsRecords,
    DomainVerificationStatus,
    Email,
    ForwardEmailRequest,
    ForwardingRule,
    ListApiKeysParams,
    ListBroadcastsParams,
    ListContactsParams,
    ListEmailsParams,
    ListSegmentsParams,
    ListTemplatesParams,
    ListWebhooksParams,
    PaginatedResponse,
    Pagination,
    ReplyTo,
    Segment,
    SegmentContactsResponse,
    SegmentCountResponse,
    SendBroadcastResponse,
    SendEmailRequest,
    SendEmailResponse,
    SuccessResponse,
    Template,
    UpdateBroadcastRequest,
    UpdateContactRequest,
    UpdateDomainRequest,
    UpdateSegmentRequest,
    UpdateTemplateRequest,
    UpdateWebhookRequest,
    Webhook,
    WebhookDelivery,
    WebhookToggleResponse,
)

__version__ = "1.0.0"

__all__ = [
    # Clients
    "Emailr",
    "AsyncEmailr",
    # Errors
    "EmailrError",
    "NetworkError",
    "AuthenticationError",
    "RateLimitError",
    "NotFoundError",
    "ValidationError",
    # Types - Common
    "PaginatedResponse",
    "Pagination",
    "SuccessResponse",
    # Types - Email
    "Attachment",
    "ReplyTo",
    "SendEmailRequest",
    "SendEmailResponse",
    "Email",
    "ListEmailsParams",
    "ForwardEmailRequest",
    "ForwardingRule",
    "CreateForwardingRuleRequest",
    # Types - Contact
    "Contact",
    "CreateContactRequest",
    "UpdateContactRequest",
    "ListContactsParams",
    "ContactListResponse",
    "BulkCreateContactsRequest",
    "BulkCreateContactsResponse",
    # Types - Template
    "Template",
    "CreateTemplateRequest",
    "UpdateTemplateRequest",
    "ListTemplatesParams",
    # Types - Domain
    "DnsRecord",
    "DomainDnsRecords",
    "Domain",
    "AddDomainRequest",
    "UpdateDomainRequest",
    "DomainVerificationStatus",
    "DnsVerificationStatus",
    # Types - Webhook
    "Webhook",
    "CreateWebhookRequest",
    "UpdateWebhookRequest",
    "WebhookToggleResponse",
    "WebhookDelivery",
    "ListWebhooksParams",
    # Types - Broadcast
    "Broadcast",
    "BroadcastStats",
    "CreateBroadcastRequest",
    "UpdateBroadcastRequest",
    "SendBroadcastResponse",
    "ListBroadcastsParams",
    # Types - Segment
    "Segment",
    "SegmentContactsResponse",
    "SegmentCountResponse",
    "CreateSegmentRequest",
    "UpdateSegmentRequest",
    "ListSegmentsParams",
    # Types - API Key
    "ApiKey",
    "CreateApiKeyRequest",
    "ListApiKeysParams",
]



"""
Webhook resource for managing webhooks.
"""

from typing import Any, Dict, List, Optional

from emailr.http import AsyncHttpClient, HttpClient
from emailr.types import (
    CreateWebhookRequest,
    PaginatedResponse,
    Pagination,
    SuccessResponse,
    UpdateWebhookRequest,
    Webhook,
    WebhookDelivery,
    WebhookToggleResponse,
)


class WebhooksResource:
    """Synchronous webhook resource for managing webhooks."""

    def __init__(self, http: HttpClient) -> None:
        self._http = http

    def create(
        self,
        name: str,
        url: str,
        events: List[str],
    ) -> Webhook:
        """
        Create a new webhook.

        Args:
            name: Webhook name
            url: Webhook URL
            events: List of events to subscribe to

        Returns:
            Created webhook
        """
        request = CreateWebhookRequest(name=name, url=url, events=events)
        data = self._http.post("/v1/webhooks", request.to_dict())
        return Webhook.from_dict(data)

    def get(self, id: str) -> Webhook:
        """
        Get webhook by ID.

        Args:
            id: Webhook ID

        Returns:
            Webhook record
        """
        data = self._http.get(f"/v1/webhooks/{id}")
        return Webhook.from_dict(data)

    def list(
        self,
        *,
        page: Optional[int] = None,
        limit: Optional[int] = None,
    ) -> List[Webhook]:
        """
        List webhooks.

        Args:
            page: Page number
            limit: Number of items per page

        Returns:
            List of webhooks
        """
        params: Dict[str, Any] = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit

        data = self._http.get("/v1/webhooks", params=params)
        # API returns array directly
        if isinstance(data, list):
            return [Webhook.from_dict(w) for w in data]
        # Handle paginated response format
        return [Webhook.from_dict(w) for w in data.get("data", data)]


    def update(
        self,
        id: str,
        *,
        name: Optional[str] = None,
        url: Optional[str] = None,
        events: Optional[List[str]] = None,
    ) -> Webhook:
        """
        Update a webhook.

        Args:
            id: Webhook ID
            name: New webhook name
            url: New webhook URL
            events: New list of events

        Returns:
            Updated webhook
        """
        request = UpdateWebhookRequest(name=name, url=url, events=events)
        data = self._http.put(f"/v1/webhooks/{id}", request.to_dict())
        return Webhook.from_dict(data)

    def delete(self, id: str) -> SuccessResponse:
        """
        Delete a webhook.

        Args:
            id: Webhook ID

        Returns:
            Success response
        """
        data = self._http.delete(f"/v1/webhooks/{id}")
        return SuccessResponse(success=data.get("success", True))

    def enable(self, id: str) -> WebhookToggleResponse:
        """
        Enable a webhook.

        Args:
            id: Webhook ID

        Returns:
            Toggle response with new active status
        """
        data = self._http.post(f"/v1/webhooks/{id}/enable")
        return WebhookToggleResponse.from_dict(data)

    def disable(self, id: str) -> WebhookToggleResponse:
        """
        Disable a webhook.

        Args:
            id: Webhook ID

        Returns:
            Toggle response with new active status
        """
        data = self._http.post(f"/v1/webhooks/{id}/disable")
        return WebhookToggleResponse.from_dict(data)

    def list_deliveries(self, id: str) -> List[WebhookDelivery]:
        """
        List webhook deliveries.

        Args:
            id: Webhook ID

        Returns:
            List of webhook deliveries
        """
        data = self._http.get(f"/v1/webhooks/{id}/deliveries")
        # API returns array directly
        if isinstance(data, list):
            return [WebhookDelivery.from_dict(d) for d in data]
        return [WebhookDelivery.from_dict(d) for d in data.get("data", data)]

    def retry_delivery(self, webhook_id: str, delivery_id: str) -> SuccessResponse:
        """
        Retry a failed webhook delivery.

        Args:
            webhook_id: Webhook ID
            delivery_id: Delivery ID

        Returns:
            Success response
        """
        data = self._http.post(f"/v1/webhooks/{webhook_id}/deliveries/{delivery_id}/retry")
        return SuccessResponse(success=data.get("success", True))


class AsyncWebhooksResource:
    """Asynchronous webhook resource for managing webhooks."""

    def __init__(self, http: AsyncHttpClient) -> None:
        self._http = http

    async def create(
        self,
        name: str,
        url: str,
        events: List[str],
    ) -> Webhook:
        """Create a new webhook."""
        request = CreateWebhookRequest(name=name, url=url, events=events)
        data = await self._http.post("/v1/webhooks", request.to_dict())
        return Webhook.from_dict(data)

    async def get(self, id: str) -> Webhook:
        """Get webhook by ID."""
        data = await self._http.get(f"/v1/webhooks/{id}")
        return Webhook.from_dict(data)

    async def list(
        self,
        *,
        page: Optional[int] = None,
        limit: Optional[int] = None,
    ) -> List[Webhook]:
        """List webhooks."""
        params: Dict[str, Any] = {}
        if page is not None:
            params["page"] = page
        if limit is not None:
            params["limit"] = limit

        data = await self._http.get("/v1/webhooks", params=params)
        # API returns array directly
        if isinstance(data, list):
            return [Webhook.from_dict(w) for w in data]
        return [Webhook.from_dict(w) for w in data.get("data", data)]

    async def update(
        self,
        id: str,
        *,
        name: Optional[str] = None,
        url: Optional[str] = None,
        events: Optional[List[str]] = None,
    ) -> Webhook:
        """Update a webhook."""
        request = UpdateWebhookRequest(name=name, url=url, events=events)
        data = await self._http.put(f"/v1/webhooks/{id}", request.to_dict())
        return Webhook.from_dict(data)

    async def delete(self, id: str) -> SuccessResponse:
        """Delete a webhook."""
        data = await self._http.delete(f"/v1/webhooks/{id}")
        return SuccessResponse(success=data.get("success", True))

    async def enable(self, id: str) -> WebhookToggleResponse:
        """Enable a webhook."""
        data = await self._http.post(f"/v1/webhooks/{id}/enable")
        return WebhookToggleResponse.from_dict(data)

    async def disable(self, id: str) -> WebhookToggleResponse:
        """Disable a webhook."""
        data = await self._http.post(f"/v1/webhooks/{id}/disable")
        return WebhookToggleResponse.from_dict(data)

    async def list_deliveries(self, id: str) -> List[WebhookDelivery]:
        """List webhook deliveries."""
        data = await self._http.get(f"/v1/webhooks/{id}/deliveries")
        # API returns array directly
        if isinstance(data, list):
            return [WebhookDelivery.from_dict(d) for d in data]
        return [WebhookDelivery.from_dict(d) for d in data.get("data", data)]

    async def retry_delivery(self, webhook_id: str, delivery_id: str) -> SuccessResponse:
        """Retry a failed webhook delivery."""
        data = await self._http.post(
            f"/v1/webhooks/{webhook_id}/deliveries/{delivery_id}/retry"
        )
        return SuccessResponse(success=data.get("success", True))



"""
Contact resource for managing contacts.
"""

from typing import Any, Dict, List, Optional

from emailr.http import AsyncHttpClient, HttpClient
from emailr.types import (
    BulkCreateContactsRequest,
    BulkCreateContactsResponse,
    Contact,
    ContactListResponse,
    CreateContactRequest,
    SuccessResponse,
    UpdateContactRequest,
)


class ContactsResource:
    """Synchronous contact resource for managing contacts."""

    def __init__(self, http: HttpClient) -> None:
        self._http = http

    def create(
        self,
        email: str,
        *,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        subscribed: Optional[bool] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> Contact:
        """
        Create a new contact.

        Args:
            email: Contact email address
            first_name: Contact first name
            last_name: Contact last name
            subscribed: Whether contact is subscribed
            metadata: Custom metadata

        Returns:
            Created contact
        """
        request = CreateContactRequest(
            email=email,
            first_name=first_name,
            last_name=last_name,
            subscribed=subscribed,
            metadata=metadata,
        )
        data = self._http.post("/v1/contacts", request.to_dict())
        return Contact.from_dict(data)

    def get(self, id: str) -> Contact:
        """
        Get contact by ID.

        Args:
            id: Contact ID

        Returns:
            Contact record
        """
        data = self._http.get(f"/v1/contacts/{id}")
        return Contact.from_dict(data)


    def list(
        self,
        *,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        subscribed: Optional[bool] = None,
    ) -> ContactListResponse:
        """
        List contacts with optional filtering.

        Args:
            limit: Number of items to return
            offset: Number of items to skip
            subscribed: Filter by subscription status

        Returns:
            Contact list response with pagination info
        """
        params: Dict[str, Any] = {}
        if limit is not None:
            params["limit"] = limit
        if offset is not None:
            params["offset"] = offset
        if subscribed is not None:
            params["subscribed"] = subscribed

        data = self._http.get("/v1/contacts", params=params)
        return ContactListResponse.from_dict(data)

    def update(
        self,
        id: str,
        *,
        email: Optional[str] = None,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        subscribed: Optional[bool] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> Contact:
        """
        Update a contact.

        Args:
            id: Contact ID
            email: New email address
            first_name: New first name
            last_name: New last name
            subscribed: New subscription status
            metadata: New metadata

        Returns:
            Updated contact
        """
        request = UpdateContactRequest(
            email=email,
            first_name=first_name,
            last_name=last_name,
            subscribed=subscribed,
            metadata=metadata,
        )
        data = self._http.put(f"/v1/contacts/{id}", request.to_dict())
        return Contact.from_dict(data)

    def delete(self, id: str) -> SuccessResponse:
        """
        Delete a contact.

        Args:
            id: Contact ID

        Returns:
            Success response
        """
        data = self._http.delete(f"/v1/contacts/{id}")
        return SuccessResponse(success=data.get("success", True))

    def bulk_create(self, contacts: List[CreateContactRequest]) -> BulkCreateContactsResponse:
        """
        Bulk create contacts.

        Args:
            contacts: List of contacts to create

        Returns:
            Bulk create response with import count
        """
        request = BulkCreateContactsRequest(contacts=contacts)
        data = self._http.post("/v1/contacts/bulk", request.to_dict())
        return BulkCreateContactsResponse.from_dict(data)

    def unsubscribe(self, id: str) -> SuccessResponse:
        """
        Unsubscribe a contact.

        Args:
            id: Contact ID

        Returns:
            Success response
        """
        data = self._http.post(f"/v1/contacts/{id}/unsubscribe")
        return SuccessResponse(success=data.get("success", True))

    def resubscribe(self, id: str) -> SuccessResponse:
        """
        Resubscribe a contact.

        Args:
            id: Contact ID

        Returns:
            Success response
        """
        data = self._http.post(f"/v1/contacts/{id}/resubscribe")
        return SuccessResponse(success=data.get("success", True))


class AsyncContactsResource:
    """Asynchronous contact resource for managing contacts."""

    def __init__(self, http: AsyncHttpClient) -> None:
        self._http = http

    async def create(
        self,
        email: str,
        *,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        subscribed: Optional[bool] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> Contact:
        """Create a new contact."""
        request = CreateContactRequest(
            email=email,
            first_name=first_name,
            last_name=last_name,
            subscribed=subscribed,
            metadata=metadata,
        )
        data = await self._http.post("/v1/contacts", request.to_dict())
        return Contact.from_dict(data)

    async def get(self, id: str) -> Contact:
        """Get contact by ID."""
        data = await self._http.get(f"/v1/contacts/{id}")
        return Contact.from_dict(data)

    async def list(
        self,
        *,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        subscribed: Optional[bool] = None,
    ) -> ContactListResponse:
        """List contacts with optional filtering."""
        params: Dict[str, Any] = {}
        if limit is not None:
            params["limit"] = limit
        if offset is not None:
            params["offset"] = offset
        if subscribed is not None:
            params["subscribed"] = subscribed

        data = await self._http.get("/v1/contacts", params=params)
        return ContactListResponse.from_dict(data)

    async def update(
        self,
        id: str,
        *,
        email: Optional[str] = None,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        subscribed: Optional[bool] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> Contact:
        """Update a contact."""
        request = UpdateContactRequest(
            email=email,
            first_name=first_name,
            last_name=last_name,
            subscribed=subscribed,
            metadata=metadata,
        )
        data = await self._http.put(f"/v1/contacts/{id}", request.to_dict())
        return Contact.from_dict(data)

    async def delete(self, id: str) -> SuccessResponse:
        """Delete a contact."""
        data = await self._http.delete(f"/v1/contacts/{id}")
        return SuccessResponse(success=data.get("success", True))

    async def bulk_create(
        self, contacts: List[CreateContactRequest]
    ) -> BulkCreateContactsResponse:
        """Bulk create contacts."""
        request = BulkCreateContactsRequest(contacts=contacts)
        data = await self._http.post("/v1/contacts/bulk", request.to_dict())
        return BulkCreateContactsResponse.from_dict(data)

    async def unsubscribe(self, id: str) -> SuccessResponse:
        """Unsubscribe a contact."""
        data = await self._http.post(f"/v1/contacts/{id}/unsubscribe")
        return SuccessResponse(success=data.get("success", True))

    async def resubscribe(self, id: str) -> SuccessResponse:
        """Resubscribe a contact."""
        data = await self._http.post(f"/v1/contacts/{id}/resubscribe")
        return SuccessResponse(success=data.get("success", True))
